@@ -1,7 +1,12 @@
 ;;; examples of extensions to Snd's graphics
 
-(define (display-colored-samples color beg dur snd chn)
-  "(display-colored-samples color beg dur snd chn) displays samples from beg for dur in color \
+(use-modules (ice-9 optargs))
+
+(provide 'snd-draw.scm)
+(if (not (provided? 'snd-extensions.scm)) (load-from-path "extensions.scm"))
+
+(define* (display-colored-samples color beg dur #:optional snd chn)
+  "(display-colored-samples color beg dur snd chn) displays samples from beg for dur in color 
 whenever they're in the current view."
   (let ((left (left-sample snd chn))
 	(right (right-sample snd chn))
@@ -14,7 +19,7 @@ whenever they're in the current view."
 	      (let* ((samps (- (min right end) (max left beg)))
 		     (offset (max 0 (- beg left)))
 		     (new-data (vct-subseq data offset (+ offset samps))))
-		(set! (foreground-color snd chn) red)
+		(set! (foreground-color snd chn) color)
 		(graph-data new-data snd chn copy-context (max beg left) (min end right))
 		(set! (foreground-color snd chn) old-color))
 	      (let* ((low-data (car data))
@@ -22,16 +27,16 @@ whenever they're in the current view."
 		     (size (vct-length low-data))
 		     (samps (- right left))
 		     (left-offset (max 0 (- beg left)))
-		     (left-bin (inexact->exact (/ (* size left-offset) samps)))
+		     (left-bin (inexact->exact (floor (/ (* size left-offset) samps))))
 		     (right-offset (- (min end right) left))
-		     (right-bin (inexact->exact (/ (* size right-offset) samps)))
+		     (right-bin (inexact->exact (floor (/ (* size right-offset) samps))))
 		     (new-low-data (vct-subseq low-data left-bin right-bin))
 		     (new-high-data (vct-subseq high-data left-bin right-bin)))
-		(set! (foreground-color snd chn) red)
+		(set! (foreground-color snd chn) color)
 		(graph-data (list new-low-data new-high-data) snd chn copy-context left-bin right-bin)
 		(set! (foreground-color snd chn) old-color)))))))
 
-(define (display-samples-in-color snd chn)
+(define* (display-samples-in-color snd chn)
   ;; intended as after-graph-hook member 
   ;; run through 'colored-samples lists passing each to display-colored-samples
   (let ((colors (channel-property 'colored-samples snd chn)))
@@ -42,21 +47,21 @@ whenever they're in the current view."
 	 colors))))
 
 (define* (color-samples color #:optional ubeg udur usnd uchn)
-  "(color-samples color &optional beg dur snd chn) causes samples from beg to beg+dur to be displayed in color"
+  "(color-samples color #:optional beg dur snd chn) causes samples from beg to beg+dur to be displayed in color"
   (if (not (member display-samples-in-color (hook->list after-graph-hook)))
       (add-hook! after-graph-hook display-samples-in-color))
   (let* ((beg (or ubeg 0))
-	 (snd (or usnd (selected-sound)))
-	 (chn (or uchn (selected-channel snd)))
+	 (snd (or usnd (selected-sound) (car (sounds))))
+	 (chn (or uchn (selected-channel snd) 0))
 	 (dur (or udur (- (frames snd chn) beg)))
 	 (old-colors (or (channel-property 'colored-samples snd chn) '())))
     (set! (channel-property 'colored-samples snd chn) (cons (list color beg dur) old-colors))
     (update-time-graph snd chn)))
 
 (define* (uncolor-samples #:optional usnd uchn)
-  "(uncolor-samples &optional snd chn) cancels sample coloring in the given channel"
-  (let*	((snd (or usnd (selected-sound)))
-	 (chn (or uchn (selected-channel snd))))
+  "(uncolor-samples #:optional snd chn) cancels sample coloring in the given channel"
+  (let*	((snd (or usnd (selected-sound) (car (sounds))))
+	 (chn (or uchn (selected-channel snd) 0)))
     (set! (channel-property 'colored-samples snd chn) '())
     (update-time-graph snd chn)))
 
@@ -107,7 +112,7 @@ whenever they're in the current view."
 
 
 (define (display-energy snd chn)
-  "(display-energy snd chn) is a lisp-graph-hook function that displays the time domain data as energy in the lisp graph; \
+  "(display-energy snd chn) is a lisp-graph-hook function that displays the time domain data as energy in the lisp graph; 
 the y-zoom-slider controls the graph amp"
   (let* ((ls (left-sample))
 	 (rs (right-sample))
@@ -131,15 +136,16 @@ the y-zoom-slider controls the graph amp"
 	 (samps (- right left))
 	 (x0 (x->position (/ left (srate))))
 	 (y0 (y->position (vct-ref data 0)))
-	 (colors (make-vector colormap-size #f)))
+	 (colors (make-vector (colormap-size) #f)))
     (do ((i (+ left 1) (1+ i))
 	 (j 1 (1+ j)))
 	((= i right))
       (let* ((x1 (x->position (/ i (srate))))
 	     (y1 (y->position (vct-ref data j)))
-	     (ref (inexact->exact (* colormap-size (abs (vct-ref data j)))))
+	     (x (abs (vct-ref data j)))
+	     (ref (inexact->exact (floor (* (colormap-size) x))))
 	     (color (or (vector-ref colors ref)
-			(let ((new-color (apply make-color (colormap-ref (colormap) ref))))
+			(let ((new-color (apply make-color (colormap-ref (colormap) x))))
 			  (vector-set! colors ref new-color)
 			  new-color))))
 	(set! (foreground-color snd chn) color)
@@ -157,54 +163,49 @@ the y-zoom-slider controls the graph amp"
 (define inset-width .2)
 (define inset-height .25)
 
-(if (not (defined? 'channel-property)) ; extensions.scm
-    (define channel-property
-      (make-procedure-with-setter
-       (lambda (key snd chn)
-	 (let ((data (assoc key (channel-properties snd chn))))
-	   (if data (cdr data) #f)))
-       (lambda (key snd chn new-val)
-	 (let ((old-val (assoc key (channel-properties snd chn))))
-	   (if old-val
-	       (set-cdr! old-val new-val)
-	       (set! (channel-properties snd chn) (cons (cons key new-val) (channel-properties snd chn))))
-	   new-val)))))
-
 (define (make-current-window-display)
 
+  (define (update-current-window-location snd)
+    ;; this is called before the actual update -- we need to clear the notion of edit-position to force a re-read
+    (do ((i 0 (1+ i)))
+	((= i (chans snd)))
+      (let ((vals (channel-property 'inset-envelope snd i)))
+	(if vals
+	    (list-set! vals 2 -2)))) ; set edit-position to impossible value
+    #f)
+
   (define (display-current-window-location snd chn)
     "display in upper right corner the overall current sound and where the current window fits in it"
-    (if (graph-time? snd chn)
+    (if (time-graph? snd chn)
 	(let* ((axinf (axis-info snd chn))
 	       (grf-width (list-ref axinf 12))
-	       (width (inexact->exact (* inset-width grf-width)))
+	       (width (inexact->exact (round (* inset-width grf-width))))
 	       (x-offset (inexact->exact (- grf-width width)))
 	       (grf-height (- (list-ref axinf 11) (list-ref axinf 13)))
-	       (height (inexact->exact (* inset-height grf-height)))
+	       (height (inexact->exact (round (* inset-height grf-height))))
 	       (chan-offset (- (list-ref axinf 13) 10))
-	       (y-offset (+ chan-offset (inexact->exact (/ height 2))))
+	       (y-offset (+ chan-offset (inexact->exact (round (/ height 2)))))
 	       (grf-chn (if (= (channel-style snd) channels-separate) chn 0))
+	       (new-peaks (list-ref axinf 18))
 	       (data0 #f)
 	       (data1 #f))
-	  
 	  (if (and (> width 10)
 		   (> height 10)
 		   (> (frames snd chn) 0)
 		   (or (= chn 0)
 		       (not (= (channel-style snd) channels-superimposed))))
 	      (begin
-		
 		;; draw axes around the inset graph
 		(fill-rectangle x-offset (+ chan-offset height) width 2 snd grf-chn)
 		(fill-rectangle x-offset chan-offset 2 height snd grf-chn)
 		
 		;; now show where the current window fits in this graph
-		(let ((rx (inexact->exact (* width (/ (right-sample snd chn) (frames snd chn)))))
-		      (lx (inexact->exact (* width (/ (left-sample snd chn) (frames snd chn))))))
+		(let ((rx (inexact->exact (round (* width (/ (right-sample snd chn) (frames snd chn))))))
+		      (lx (inexact->exact (round (* width (/ (left-sample snd chn) (frames snd chn)))))))
 		  (fill-rectangle (+ x-offset lx) chan-offset (max 1 (- rx lx)) height snd grf-chn selection-context))
-		
 		(let ((old-env (channel-property 'inset-envelope snd chn)))
 		  (if (and old-env
+			   (not new-peaks)
 			   (= width (car old-env))
 			   (= height (cadr old-env))
 			   (= y-offset (list-ref old-env 5))
@@ -213,12 +214,12 @@ the y-zoom-slider controls the graph amp"
 			(set! data0 (list-ref old-env 3))
 			(set! data1 (list-ref old-env 4)))
 		      (let* ((data (make-graph-data snd chn current-edit-position 0 (frames snd chn)))
-			     (data-max (if (vct? data) (vct-peak data) (vct-peak (car data))))
+			     (data-max (if (vct? data) (vct-peak data)
+					   (apply max (map vct-peak data))))
 			     (data-scaler (if (> data-max 0.0) (/ height (* 2 data-max)) 0.0))
 			     (new-len (* width 2))
 			     (data-len (if (vct? data) (vct-length data) (vct-length (car data))))
 			     (step (/ data-len width)))
-			
 			(if (> data-len width)
 			    (begin ; the normal case -- more samples to display than pixels available
 			      (set! data0 (make-vector new-len))
@@ -233,19 +234,19 @@ the y-zoom-slider controls the graph amp"
 				    ((or (= j new-len) (= i data-len)))
 				  (if data1
 				      (begin
-					(set! max-y (max max-y (vct-ref (car data) i)))
-					(set! min-y (min min-y (vct-ref (cadr data) i))))
+					(set! max-y (max max-y (vct-ref (cadr data) i)))
+					(set! min-y (min min-y (vct-ref (car data) i))))
 				      (set! max-y (max max-y (vct-ref data i))))
 				  (set! stepper (+ stepper 1.0))
 				  (if (>= stepper step)
 				      (begin
 					(vector-set! data0 j x-offset) 
-					(vector-set! data0 (+ j 1) (inexact->exact (- y-offset (* max-y data-scaler))))
+					(vector-set! data0 (+ j 1) (inexact->exact (round (- y-offset (* max-y data-scaler)))))
 					(set! max-y (- data-max))
 					(if data1
 					    (begin
 					      (vector-set! data1 j x-offset) 
-					      (vector-set! data1 (+ j 1) (inexact->exact (- y-offset (* min-y data-scaler))))
+					      (vector-set! data1 (+ j 1) (inexact->exact (round (- y-offset (* min-y data-scaler)))))
 					      (set! min-y data-max)))
 					(set! x-offset (+ x-offset 1))
 					(set! stepper (- stepper step))
@@ -267,13 +268,13 @@ the y-zoom-slider controls the graph amp"
 				   (j 0 (+ j 2))
 				   (xj x-offset (+ xj xstep)))
 				  ((= i data-len))
-				(vector-set! data0 j (inexact->exact xj))
+				(vector-set! data0 j (inexact->exact (round xj)))
 				(if (not data1)
-				    (vector-set! data0 (+ j 1) (inexact->exact (- y-offset (* (vct-ref data i) data-scaler))))
+				    (vector-set! data0 (+ j 1) (inexact->exact (round (- y-offset (* (vct-ref data i) data-scaler)))))
 				    (begin
-				      (vector-set! data0 (+ j 1) (inexact->exact (- y-offset (* (vct-ref (car data) i) data-scaler))))
-				      (vector-set! data1 j (inexact->exact xj))
-				      (vector-set! data1 (+ j 1) (inexact->exact (- y-offset (* (vct-ref (cadr data) i) data-scaler)))))))))
+				      (vector-set! data0 (+ j 1) (inexact->exact (round (- y-offset (* (vct-ref (cadr data) i) data-scaler)))))
+				      (vector-set! data1 j (inexact->exact (floor xj)))
+				      (vector-set! data1 (+ j 1) (inexact->exact (round (- y-offset (* (vct-ref (car data) i) data-scaler))))))))))
 			(set! (channel-property 'inset-envelope snd chn) 
 			      (list width height (edit-position snd chn) data0 data1 y-offset)))))
 		
@@ -284,28 +285,132 @@ the y-zoom-slider controls the graph amp"
     (if (= axis time-graph)
 	(let* ((axinf (axis-info snd chn))
 	       (grf-width (list-ref axinf 12))
-	       (width (inexact->exact (* inset-width grf-width)))
+	       (width (inexact->exact (round (* inset-width grf-width))))
 	       (x-offset (inexact->exact (- grf-width width)))
 	       (grf-height (- (list-ref axinf 11) (list-ref axinf 13)))
-	       (height (inexact->exact (* inset-height grf-height)))
+	       (height (inexact->exact (round (* inset-height grf-height))))
 	       (chan-offset (- (list-ref axinf 13) 10))
-	       (y-offset (+ chan-offset (inexact->exact (/ height 2)))))
+	       (y-offset (+ chan-offset (inexact->exact (round (/ height 2))))))
 	  (if (and (> width 0)
 		   (>= x x-offset)
 		   (<= x grf-width)
 		   (>= y chan-offset)
 		   (<= y (+ chan-offset height)))
-	      (let ((samp (inexact->exact (* (frames snd chn) (/ (- x x-offset) width)))))
+	      (let* ((samp (inexact->exact (round (* (frames snd chn) (/ (- x x-offset) width)))))
+		     (ls (left-sample snd chn))
+		     (rs (right-sample snd chn)))
 		(set! (cursor snd chn) samp)
-		(set! (right-sample snd chn) 
-		      (max 0 
-			   (min (frames snd chn)
-				(- samp (inexact->exact (* .5 (- (left-sample snd chn) (right-sample snd chn))))))))
+		(if (or (< samp ls)
+			(> samp rs))
+		    (let ((rsamp (min (max 0 
+					   (- samp (inexact->exact (round (* .5 (- ls rs))))))
+				      (1- (frames snd chn)))))
+		      (set! (right-sample snd chn) rsamp)))
 		(update-time-graph)
 		#t)
 	      #f))
 	#f))
   
+  (add-hook! after-open-hook 
+    (lambda (s)
+      (do ((i 0 (1+ i)))
+	  ((= i (chans s)))
+	(set! (channel-property 'save-state-ignore s i)
+	      (cons 'inset-envelope 
+		    (or (channel-property 'save-state-ignore s i) 
+			(list 'save-state-ignore))))
+	(add-hook! (undo-hook s i)
+		   (lambda ()
+		     (let ((vals (channel-property 'inset-envelope s i)))
+		       (if vals
+			   (list-set! vals 2 -2)))))))) ; set edit-position to impossible value
   (add-hook! after-graph-hook display-current-window-location)
-  (add-hook! mouse-click-hook click-current-window-location))
+  (add-hook! mouse-click-hook click-current-window-location)
+  (add-hook! update-hook update-current-window-location))
 
+
+;;; -------- click-for-listener-help
+
+(define click-for-listener-help
+  (let ((help-moved #f)
+	(last-click-time 0))
+    (lambda (pos)
+  
+      (define (char-quit? ch)
+	(or (char-whitespace? ch)
+	    (char=? ch #\()
+	    (char=? ch #\))
+	    (char=? ch #\')
+	    (char=? ch #\")))
+
+      ;; find nearest name
+      ;; scan back and forth for non-alphanumeric
+      ;; call snd-help on name and post in help-dialog
+      
+      (let* ((time (get-internal-real-time)) ; look for double-click
+	     (click-time (- time last-click-time)))
+	(set! last-click-time time)
+	(if (< click-time 25) ; .25 secs -- could use XtGetMultiClickTime if xm loaded
+	    (let* ((text (widget-text (list-ref (main-widgets) 4)))
+		   (len (and (string? text) (string-length text)))
+		   (happy #f))
+	      (if (and len (>= pos len)) (set! pos (1- len))) ;click at very bottom of listener is beyond current string end
+	      (if (and len (>= pos 1))                        ;not click in empty listener, etc
+		  (begin
+		    (if (char-quit? (string-ref text pos))
+			(begin                      ;not currently in a name
+			  ;; go looking for plausible name (look left first)
+			  (do ((i (1- pos) (1- i)))
+			      ((or happy (= i 0)))
+			    (if (char-alphabetic? (string-ref text i))
+				(begin
+				  (set! pos i)
+				  (set! happy #t))))
+			  (if (not happy)           ;nothing to the left -- try to the right
+			      (do ((i (1+ pos) (1+ i)))
+				  ((or happy (= i len)))
+				(if (char-alphabetic? (string-ref text i))
+				    (begin
+				      (set! pos i)
+				      (set! happy #t))))))
+			(set! happy #t))
+		    (if happy
+			;; found some portion of a name -- try to get the full thing
+			(let ((start pos)
+			      (end pos))
+			  (set! happy #f)
+			  (do ((i (1- pos) (1- i)))  ;look for start of name
+			      ((or happy (= i 0)))
+			    (if (char-quit? (string-ref text i))
+				(begin
+				  (set! start (+ i 1))
+				  (set! happy #t))))
+			  (set! happy #f)
+			  (do ((i (1+ pos) (1+ i)))  ;look for end of name
+			      ((or happy (and (= i len)
+					      (let () 
+						(set! end i) 
+						(set! happy #t) 
+						#t))))
+			    (if (char-quit? (string-ref text i))
+				(begin
+				  (set! end i)
+				  (set! happy #t))))
+			  (if (and happy
+				   (> end start))
+			      ;; got a name -- look for snd-help, if any
+			      (let* ((name (substring text start end))
+				     (help (snd-help name #f)))
+				(if (string=? name help)
+				    (set! help (snd-help (string->symbol name))))
+				(if (not (string=? name help))
+				    (let ((dialog (help-dialog name help)))
+				      (if (not help-moved)
+					  ;; if this is the first call, try to position help dialog out of the way
+					  ;;   this is clunky -- perhaps better would be a tooltip window at the bottom of the listener?
+					  (let ((main-xy (widget-position (cadr (main-widgets))))
+						(main-size (widget-size (cadr (main-widgets)))))
+					    (set! (widget-position (list-ref (dialog-widgets) 14))
+						  (list (+ (car main-xy) (car main-size))
+							(cadr main-xy)))
+					    (set! help-moved #t)))))))))))))))))
