@@ -2,6 +2,7 @@
 /*   based on code of Scott Wilson and Craig Sapp */
 
 /* TODO: test, tie into CLM (rt.lisp, sndlib2clm.lisp etc)
+ * TODO: add mus-midi support to run macro(s) [unsigned char* buf arg is a list in scheme]
  */
 
 /*
@@ -15,15 +16,13 @@
  *
  *    not implemented (not sure it's needed): int mus_midi_flush(line)
  *
- * currently OSS, ALSA, SGI (both libraries), some of OSX
+ * currently OSX, ALSA, SGI (both libraries), OSS
  */
 
-#if defined(HAVE_CONFIG_H)
-  #include "config.h"
-#endif
+#include <config.h>
 
 #include <stdio.h>
-#if (!defined(HAVE_CONFIG_H)) || (defined(HAVE_FCNTL_H))
+#if HAVE_FCNTL_H
   #include <fcntl.h>
 #endif
 #include <errno.h>
@@ -35,7 +34,7 @@
     #include <unistd.h>
   #endif
 #endif
-#if (!defined(HAVE_CONFIG_H)) || (defined(HAVE_STRING_H))
+#if HAVE_STRING_H
   #include <string.h>
 #endif
 
@@ -66,6 +65,10 @@
 #if HAVE_ALSA
 
 #include <sys/ioctl.h>
+
+#define ALSA_PCM_OLD_HW_PARAMS_API
+#define ALSA_PCM_OLD_SW_PARAMS_API
+
 #if HAVE_ALSA_ASOUNDLIB_H
   #include <alsa/asoundlib.h>
 #else
@@ -136,7 +139,7 @@ static int midi_open(const char *name, int input)
 #ifdef NO_SNDLIB
       fprintf(stderr, "can't open %s: %s\n", name, strerror(err));
 #else
-      mus_error(MUS_MIDI_OPEN_ERROR, "can't open %s: %s", name, strerror(err));
+      return(mus_error(MUS_MIDI_OPEN_ERROR, "can't open %s: %s", name, strerror(err)));
 #endif
       return(-1);
     }
@@ -147,9 +150,9 @@ static int midi_open(const char *name, int input)
       snd_rawmidi_params_free(params);
       snd_rawmidi_close(line);
 #ifdef NO_SNDLIB
-      fprintf(stderr,"can't set %s buffer size to %d: %s\n", name, DEV_BUFSIZE, strerror(err));
+      fprintf(stderr, "can't set %s buffer size to %d: %s\n", name, DEV_BUFSIZE, strerror(err));
 #else
-      mus_error(MUS_MIDI_MISC_ERROR, "can't set %s buffer size to %d: %s", name, DEV_BUFSIZE, strerror(err));
+      return(mus_error(MUS_MIDI_MISC_ERROR, "can't set %s buffer size to %d: %s", name, DEV_BUFSIZE, strerror(err)));
 #endif
       return(-1);
     }
@@ -162,7 +165,7 @@ int mus_midi_open_write(const char *name) {return(midi_open(name, MIDI_WRITE));}
 
 int mus_midi_close(int line)
 {
-  int err;
+  int err = 0;
   if ((line >= 0) && (line < midis))
     {
       free(midi_names[line]);
@@ -175,9 +178,9 @@ int mus_midi_close(int line)
   if (err)
     {
 #ifdef NO_SNDLIB
-      fprintf(stderr,"can't close %s: %s\n", midi_names[line], strerror(err));
+      fprintf(stderr, "can't close %s: %s\n", midi_names[line], strerror(err));
 #else
-      mus_error(MUS_MIDI_CLOSE_ERROR, "can't close %s: %s", midi_names[line], strerror(err));
+      return(mus_error(MUS_MIDI_CLOSE_ERROR, "can't close %s: %s", midi_names[line], strerror(err)));
 #endif
       return(-1);
     }
@@ -192,7 +195,7 @@ int mus_midi_read(int line, unsigned char *buffer, int bytes)
 #ifdef NO_SNDLIB
   fprintf(stderr, "can't read from output %s\n", midi_names[line]);
 #else
-  mus_error(MUS_MIDI_READ_ERROR, "can't read from output %s", midi_names[line]);
+  return(mus_error(MUS_MIDI_READ_ERROR, "can't read from output %s", midi_names[line]));
 #endif
   return(-1);
 }
@@ -205,7 +208,7 @@ int mus_midi_write(int line, unsigned char *buffer, int bytes)
 #ifdef NO_SNDLIB
   fprintf(stderr, "can't write to input %s\n", midi_names[line]);
 #else
-  mus_error(MUS_MIDI_WRITE_ERROR, "can't write to input %s", midi_names[line]);
+  return(mus_error(MUS_MIDI_WRITE_ERROR, "can't write to input %s", midi_names[line]));
 #endif
   return(-1);
 }
@@ -230,14 +233,12 @@ char *mus_midi_describe(void)
   buf = (char *)CALLOC(1024, sizeof(char));
   for (i = 0; i < 8; i++)
     {
-
-      /* TODO: presumably this should use the card|device from the info struct, not a blind loop */
-
+      /* presumably this should use the card|device from the info struct, not a blind loop */
       err = snd_rawmidi_open(&line, NULL, mus_midi_device_name(i << 16), SND_RAWMIDI_NONBLOCK); /* do the "devices" matter here? */
       if (err < 0) continue;
       err = snd_rawmidi_info(line, info);
       if (err < 0) break;
-      sprintf(one, "%s: card: %d, device: %d, stream: %d, flags: %x, id: %d, name: %s[%s; %d]\n",
+      sprintf(one, "%s: card: %d, device: %d, stream: %d, flags: %x, id: %s, name: %s[%s; %d]\n",
 	      mus_midi_device_name(i << 16),
 	      snd_rawmidi_info_get_card(info),    /* card number */
 	      snd_rawmidi_info_get_device(info),  /* device number */
@@ -280,7 +281,7 @@ static void mus_midi_initialize(void)
       midi_ports = mdInit();
       if (midi_ports == -1)
 #ifdef NO_SNDLIB
-	fprintf(stderr,"startmidi not called?");
+	fprintf(stderr, "startmidi not called?");
 #else
       mus_error(MUS_MIDI_INIT_ERROR, "startmidi not called?");
 #endif
@@ -332,11 +333,11 @@ int midi_open(const char *name, int input)
   if (md == NULL)
     {
 #ifdef NO_SNDLIB
-      fprintf(stderr,"can't open %s\n", name);
-#else
-      mus_error(MUS_MIDI_OPEN_ERROR, "can't open %s", name);
-#endif
+      fprintf(stderr, "can't open %s\n", name);
       return(-1);
+#else
+      return(mus_error(MUS_MIDI_OPEN_ERROR, "can't open %s", name));
+#endif
     }
   mdSetStampMode(md, MD_NOSTAMP);
   return(new_midi_line(name, md, input));
@@ -371,10 +372,10 @@ int mus_midi_read(int line, unsigned char *buffer, int bytes)
     }
 #ifdef NO_SNDLIB
   fprintf(stderr, "can't read from output %s\n", midi_names[line]);
-#else
-  mus_error(MUS_MIDI_READ_ERROR, "can't read from output %s", midi_names[line]);
-#endif
   return(-1);
+#else
+  return(mus_error(MUS_MIDI_READ_ERROR, "can't read from output %s", midi_names[line]));
+#endif
 }
 
 int mus_midi_write(int line, unsigned char *buffer, int bytes)
@@ -399,10 +400,10 @@ int mus_midi_write(int line, unsigned char *buffer, int bytes)
     }
 #ifdef NO_SNDLIB
   fprintf(stderr, "can't write to input %s\n", midi_names[line]);
-#else
-  mus_error(MUS_MIDI_WRITE_ERROR, "can't write to input %s", midi_names[line]);
-#endif
   return(-1);
+#else
+  return(mus_error(MUS_MIDI_WRITE_ERROR, "can't write to input %s", midi_names[line]));
+#endif
 }
 
 char *mus_midi_device_name(int sysdev)
@@ -453,7 +454,7 @@ char *mus_midi_describe(void)
       #if (VAR_LIB_OSS)
         #include "/var/lib/oss/include/sys/soundcard.h"
       #else
-        #if defined(HAVE_SYS_SOUNDCARD_H) || defined(LINUX) || defined(UW2)
+        #if defined(HAVE_SYS_SOUNDCARD_H) || defined(LINUX)
           #include <sys/soundcard.h>
         #else
           #if defined(HAVE_MACHINE_SOUNDCARD_H)
@@ -482,7 +483,7 @@ char *mus_midi_describe(void)
   char info[256];
   struct midi_info minfo;
   fd = open("/dev/sequencer", O_RDWR, 0);
-  if (fd == -1) fd = open("/dev/music", O_RDONLY, 0); /* /dev/sequencer never works */
+  if (fd == -1) fd = open("/dev/music", O_RDONLY, 0); 
   if (fd != -1)
     {
       status = ioctl(fd, SNDCTL_SEQ_NRMIDIS, &numdevs);
@@ -509,6 +510,7 @@ char *mus_midi_describe(void)
 		  strcat(buf, info);
 		}
 	    }
+	  close(fd);
 	}
     }
   return(buf);
@@ -533,7 +535,9 @@ char *mus_midi_describe(void)
   char name[64], mfg[64], model[64], all[192];
   char *buf;
   n = MIDIGetNumberOfDevices();
-  buf = (char *)CALLOC(n * 192, sizeof(char));
+  if (n <= 0)
+    buf = (char *)CALLOC(192, sizeof(char));
+  else buf = (char *)CALLOC(n * 192, sizeof(char));
   if (n <= 0)
     sprintf(buf, "no midi");
   else
@@ -607,7 +611,6 @@ int mus_midi_read(int line, unsigned char *buf, int bytes)
 static Byte bb[1024];
 int mus_midi_write(int line, unsigned char *buffer, int bytes)
 {
-  int i;
   MIDIPacketList *list = (MIDIPacketList *)bb;
   MIDIPacket *pk;
   if ((line < 1) || (buffer == NULL) || (outp == NULL)) return(-1);
@@ -615,6 +618,7 @@ int mus_midi_write(int line, unsigned char *buffer, int bytes)
   pk = MIDIPacketListInit(list);
   MIDIPacketListAdd(list, sizeof(bb), pk, 0, bytes, (Byte *)buffer);  /* 0 is the time stamp = now? */
   MIDISend(outp, MIDIGetDestination(0), list);
+  return(0);
 }
 
 int mus_midi_open_read(const char *name)
@@ -664,6 +668,8 @@ char *mus_midi_describe(void) {return(NULL);}            /* result should be fre
 /* ---------------- XEN ---------------- */
 /* tie foregoing into Xen (Guile or Ruby) */
 
+/* TODO: mus-midi example in scm */
+
 #if HAVE_EXTENSION_LANGUAGE
 
 #include "xen.h"
@@ -678,28 +684,28 @@ char *mus_midi_describe(void) {return(NULL);}            /* result should be fre
 
 static XEN g_mus_midi_open_read(XEN name)
 {
-  #define H_mus_midi_open_read "(" S_mus_midi_open_read " name) opens midi input port, returns int id"
+  #define H_mus_midi_open_read "(" S_mus_midi_open_read " name): opens midi input port, returns int id"
   XEN_ASSERT_TYPE(XEN_STRING_P(name), name, XEN_ONLY_ARG, S_mus_midi_open_read, "a string");
   return(C_TO_XEN_INT(mus_midi_open_read(XEN_TO_C_STRING(name))));
 }
 
 static XEN g_mus_midi_open_write(XEN name)
 {
-  #define H_mus_midi_open_write "(" S_mus_midi_open_write " name) opens midi output port, returns int id"
+  #define H_mus_midi_open_write "(" S_mus_midi_open_write " name): opens midi output port, returns int id"
   XEN_ASSERT_TYPE(XEN_STRING_P(name), name, XEN_ONLY_ARG, S_mus_midi_open_write, "a string");
   return(C_TO_XEN_INT(mus_midi_open_write(XEN_TO_C_STRING(name))));
 }
 
 static XEN g_mus_midi_close(XEN line)
 {
-  #define H_mus_midi_close "(" S_mus_midi_close " line) closes midi port"
+  #define H_mus_midi_close "(" S_mus_midi_close " line): closes midi port"
   XEN_ASSERT_TYPE(XEN_INTEGER_P(line), line, XEN_ONLY_ARG, S_mus_midi_close, "an integer");
   return(C_TO_XEN_INT(mus_midi_close(XEN_TO_C_INT(line))));
 }
 
 static XEN g_mus_midi_read(XEN line, XEN bytes) /* returns list of midi bytes */
 {
-  #define H_mus_midi_read "(" S_mus_midi_read " line bytes) reads bytes from midi port, returns list"
+  #define H_mus_midi_read "(" S_mus_midi_read " line bytes): reads bytes from midi port, returns list"
   int i, len, err;
   unsigned char *buf;
   XEN lst = XEN_EMPTY_LIST;
@@ -721,7 +727,7 @@ static XEN g_mus_midi_read(XEN line, XEN bytes) /* returns list of midi bytes */
 
 static XEN g_mus_midi_write(XEN line, XEN buffer)
 {
-  #define H_mus_midi_write "(" S_mus_midi_write " line byte-list) sends byte-list to midi port"
+  #define H_mus_midi_write "(" S_mus_midi_write " line byte-list): sends byte-list to midi port"
   int i, len, err;
   unsigned char *buf;
   XEN lst;
@@ -741,14 +747,14 @@ static XEN g_mus_midi_write(XEN line, XEN buffer)
 static XEN g_mus_midi_device_name(XEN dev)
 {
   /* sndlib style sys|dev packing, dev optional */
-  #define H_mus_midi_device_name "(" S_mus_midi_device_name " &optional sys-dev) returns a name suitable for " S_mus_midi_open_read " and write"
+  #define H_mus_midi_device_name "(" S_mus_midi_device_name " (sys-dev 0)): returns a name suitable for " S_mus_midi_open_read " and write"
   XEN_ASSERT_TYPE(XEN_INTEGER_IF_BOUND_P(dev), dev, XEN_ONLY_ARG, S_mus_midi_device_name, "and integer");
   return(C_TO_XEN_STRING(mus_midi_device_name(XEN_TO_C_INT_OR_ELSE(dev, 0))));
 }
 
 static XEN g_mus_midi_describe(void)
 {
-  #define H_mus_midi_describe "(" S_mus_midi_describe ") returns a description of the midi hardware"
+  #define H_mus_midi_describe "(" S_mus_midi_describe "): returns a description of the midi hardware"
   char *str;
   XEN res = XEN_FALSE;
   str = mus_midi_describe();
@@ -778,7 +784,11 @@ static XEN g_mus_midi_describe(void)
   #define g_mus_midi_describe_w g_mus_midi_describe
 #endif
 
+#if WITH_MODULES
+static void midi_init(void *ignore)
+#else
 void mus_midi_init(void)
+#endif
 {
   XEN_DEFINE_PROCEDURE(S_mus_midi_open_read,   g_mus_midi_open_read_w, 1, 0, 0,   H_mus_midi_open_read);
   XEN_DEFINE_PROCEDURE(S_mus_midi_open_write,  g_mus_midi_open_write_w, 1, 0, 0,  H_mus_midi_open_write);
@@ -787,6 +797,23 @@ void mus_midi_init(void)
   XEN_DEFINE_PROCEDURE(S_mus_midi_write,       g_mus_midi_write_w, 2, 0, 0,       H_mus_midi_write);
   XEN_DEFINE_PROCEDURE(S_mus_midi_device_name, g_mus_midi_device_name_w, 0, 1, 0, H_mus_midi_device_name);
   XEN_DEFINE_PROCEDURE(S_mus_midi_describe,    g_mus_midi_describe_w, 0, 0, 0,    H_mus_midi_describe);
+
+#if WITH_MODULES
+  scm_c_export(S_mus_midi_open_read,
+	       S_mus_midi_open_write,
+	       S_mus_midi_close,
+	       S_mus_midi_read,
+	       S_mus_midi_write,
+	       S_mus_midi_device_name,
+	       S_mus_midi_describe,
+	       NULL);
+#endif
 }
 
+#if WITH_MODULES
+void mus_midi_init(void)
+{
+  scm_c_define_module("snd sndlib", midi_init, NULL);  
+}
+#endif
 #endif
