@@ -3,27 +3,25 @@
 ;;; play-sound #:optional func -- play current sound, calling (func data) on each buffer if func is passed
 ;;; play-often, play-until-c-g -- play sound n times or until c-g
 ;;; play-region-forever -- play region over and over until C-g typed
-;;; play samples created on-the-fly
 ;;; loop-between-marks -- play while looping continuously between two movable marks
 ;;; start-dac -- hold DAC open and play sounds via keyboard
 ;;; "vector synthesis"
 ;;; play-with-amps -- play channels with individually settable amps
-
+;;; play-sine and play-sines -- produce tones direct to DAC
 
 ;;; see also play-syncd-marks and play-between-marks in marks.scm
 
 
 (use-modules (ice-9 format) (ice-9 optargs))
+(provide 'snd-play.scm)
 
-(define* (play-sound #:optional func)
-  "(play-sound &optional func) plays the currently selected sound, calling func on each data buffer, if func exists"
-  ;;   this is essentially what the built-in play function is doing
-  ;;
-  ;; first try the simple case(s) -- most sound systems can handle short (16-bit) data and 1 or 2 channels
-  (let* ((filechans (chans))       ; initial settings are picked up from the current sound
-	 (outchans filechans)
-	 (size 256)
-	 (outbytes (* size 2))     ; 2 here since we'll first try to send short (16-bit) data to the DAC
+(define* (open-play-output #:optional out-chans out-srate out-format out-bufsize)
+  ;; returns (list audio-fd chans frames)
+  (let* ((outchans (or out-chans 1))
+	 (cur-srate (or out-srate (and (not (null? (sounds))) (srate)) 22050))
+	 (pframes (or out-bufsize 256))
+	 (frm (or out-format (if (little-endian?) mus-lshort mus-bshort)))
+	 (outbytes (* pframes 2))     ; 2 here since we'll first try to send short (16-bit) data to the DAC
 	 (audio-fd ;; ALSA throws an error where the rest of the audio cases simply report failure
 	           ;;   so we turn off the "error" printout, catch the error itself, and toss it
 	  (let ((no-error (hook-empty? mus-error-hook)))
@@ -31,9 +29,7 @@
 		(add-hook! mus-error-hook (lambda (typ msg) #t)))
 	    (let ((val (catch #t
 			      (lambda ()
-				(mus-audio-open-output mus-audio-default (srate) filechans 
-						       (if (little-endian?) mus-lshort mus-bshort) 
-						       outbytes))
+				(mus-audio-open-output mus-audio-default cur-srate outchans frm outbytes))
 			      (lambda args -1)))) ; -1 returned in case of error
 	      (if no-error 
 		  (reset-hook! mus-error-hook))
@@ -47,108 +43,91 @@
 	    (set! outchans (inexact->exact (vector-ref vals 0)))
 	    (let ((err (mus-audio-mixer-read mus-audio-default mus-audio-samples-per-channel 2 vals)))
 	      (if (not (= err -1))
-		  (set! size (inexact->exact (vector-ref vals 0))))
-	      (let* ((bps (mus-data-format-bytes-per-sample fmt)))
-		(set! outbytes (* bps size outchans))
-		(set! audio-fd (mus-audio-open-output mus-audio-default (srate) outchans fmt outbytes)))))))
-    ;; it's still possible the srate is unplayable -- Mac OSX only accepts stereo 44100!
-    ;; (snd-print (format #f "fd: ~A, size: ~A, outchans: ~A, filechans: ~A~%" audio-fd size outchans filechans))
+		  (set! pframes (inexact->exact (vector-ref vals 0))))
+	      (let* ((bps (mus-bytes-per-sample fmt)))
+		(set! outbytes (* bps pframes outchans))
+		(set! audio-fd (catch #t
+				      (lambda ()
+					(mus-audio-open-output mus-audio-default cur-srate outchans fmt outbytes))
+				      (lambda args -1))))))))
+    (if (not (= audio-fd -1))
+	(set! (dac-size) outbytes))
+    (list audio-fd outchans pframes)))
+
+(define* (play-sound #:optional func)
+  "(play-sound #:optional func) plays the currently selected sound, calling func on each data buffer, if func exists"
+  (let* ((filechans (chans))
+	 (audio-info (open-play-output filechans (srate) #f 256))
+	 (audio-fd (car audio-info))
+	 (outchans (cadr audio-info))
+	 (pframes (caddr audio-info)))
     (if (not (= audio-fd -1))
 	(let ((len (frames))
-	      (data (make-sound-data outchans size)))  ; the data buffer passed to the function (func above), then to mus-audio-write
-	  (set! (dac-size) outbytes)
-	  (do ((beg 0 (+ beg size)))
+	      (data (make-sound-data outchans pframes)))  ; the data buffer passed to the function (func above), then to mus-audio-write
+	  (do ((beg 0 (+ beg pframes)))
 	      ((or (c-g?)                   ; C-g to stop in mid-stream
 		   (> beg len)))
-	    (if (> filechans 1)
+	    (if (and (> outchans 1) (> filechans 1))
 		(do ((k 0 (1+ k)))
-		    ((= k filechans))
-		  (samples->sound-data beg size #f k data current-edit-position k))
-		(samples->sound-data beg size #f 0 data))
+		    ((= k (min outchans filechans)))
+		  (samples->sound-data beg pframes #f k data current-edit-position k))
+		(samples->sound-data beg pframes #f 0 data))
 	    (if func
 		(func data))
-	    (mus-audio-write audio-fd data size))
+	    (mus-audio-write audio-fd data pframes))
 	  (mus-audio-close audio-fd))
 	(snd-print "could not open dac"))))
 
+#!
+(play-sound 
+ (lambda (data)
+   (let ((len (sound-data-length data)))
+     (do ((i 0 (1+ i)))
+	 ((= i len))
+       (sound-data-set! data 0 i (* 2.0 (sound-data-ref data 0 i)))))))
+!#
+
 
 ;;; -------- play sound n times -- (play-often 3) for example.
 
 (define (play-often n) 
   "(play-often n) plays the selected sound 'n' times (interruptible via C-g)"
   (let ((plays (- n 1)))
-    (define (play-once snd)
-      (if (or (= plays 0)
-	      (c-g?))
-	  (remove-hook! stop-playing-hook play-once)
+    (define (play-once reason)
+      (if (and (> plays 0)
+	       (not (c-g?))
+	       (= reason 0))
 	  (begin
 	    (set! plays (- plays 1))
-	    (play 0 snd))))
-    (add-hook! stop-playing-hook play-once)
-    (play)))
+	    (play 0 #f #f #f #f #f play-once))))
+    (play 0 #f #f #f #f #f play-once)))
 
-;(bind-key (char->integer #\p) 0 (lambda (n) (play-often (max 1 n))))
+;(bind-key (char->integer #\p) 0 (lambda (n) "play often" (play-often (max 1 n))))
 
 
 ;;; -------- play sound until c-g
 
 (define (play-until-c-g)
   "(play-until-c-g) plays the selected sound until you interrupt it via C-g"
-  (define (play-once snd)
-    (if (c-g?)
-	(remove-hook! stop-playing-hook play-once)
-	(play 0 snd)))
-  (add-hook! stop-playing-hook play-once)
-  (play))
+  (define (play-once reason)
+    (if (and (not (c-g?))
+	     (= reason 0))
+	(play 0 #f #f #f #f #f play-once)))
+  (play 0 #f #f #f #f #f play-once))
 
 
 ;;; -------- play region over and over until C-g typed
 
 (define (play-region-forever reg)
   "(play-region-forever reg) plays region 'reg' until you interrupt it via C-g"
-  (define (play-region-again reg)
-    (if (c-g?)
-	(remove-hook! stop-playing-region-hook play-region-again)
-	(play-region reg)))
-  (reset-hook! stop-playing-region-hook)
-  (add-hook! stop-playing-region-hook play-region-again)
-  (play-region reg))
-
-;(bind-key (char->integer #\p) 0 (lambda (n) (play-region-forever (list-ref (regions) (max 0 n)))))
+  (define (play-region-again reason)
+    (if (and (not (c-g?))  ; be extra careful (probably superfluous)
+	     (= reason 0)) ; 0=play completed normally
+	(play-region reg #f play-region-again)))
+  (play-region reg #f play-region-again))
 
 
-#!
-;;; these are obsolete now
-;;; -------- play samples created on-the-fly
-
-(define (play-fun proc bufsize)
-  "(play-fun proc bufsize) send the output of 'proc' to the DAC.  proc should return #t to indicate completion"
-  (let* ((data (make-sound-data 1 bufsize))
-	 (bytes (* bufsize 2))
-	 (audio-fd (mus-audio-open-output mus-audio-default 22050 1 mus-lshort bytes)))
-    (if (not (= audio-fd -1))
-	(begin
-	  (do ((result #f (proc audio-fd data bufsize))) (result))
-	  (mus-audio-close audio-fd)))))
-
-(define (ampit len scaler)
-  "(ampit len scaler) plays (via play-fun) the selected sound scaled by scaler: (play-fun (ampit (frames) 2.0) 256)"
-  (let ((beg 0))
-    (lambda (fd data size) 
-      (vct->sound-data (vct-scale! (samples->vct beg size) scaler) data 0)
-      (mus-audio-write fd data size)
-      (set! beg (+ beg size))
-      (>= beg len))))
-
-(define (amprt len)
-  "(amprt len) plays (via play-fun) the selected sound scaled by the control panel amp slider: (play-fun (amprt (frames)) 256)"
-  (let ((beg 0))
-    (lambda (fd data size) 
-      (vct->sound-data (vct-scale! (samples->vct beg size) (amp)) data 0)
-      (mus-audio-write fd data size)
-      (set! beg (+ beg size))
-      (or (c-g?) (>= beg len)))))
-!#
+;(bind-key (char->integer #\p) 0 (lambda (n) "play region forever" (play-region-forever (list-ref (regions) (max 0 n)))))
 
 
 ;;; -------- play while looping continuously between two movable marks
@@ -182,7 +161,7 @@
 		  (set! end (max pos1 pos2)))))))))
 
 ;;; m1 and m2 are mark (id) numbers
-;;; (loopit 0 1 512)
+;;; (loop-between-marks 0 1 512)
 
 
 
@@ -200,8 +179,8 @@
 
 (define stop-dac stop-playing)
 
-;(bind-key (char->integer #\o) 0 (lambda () (play "oboe.snd")))
-;(bind-key (char->integer #\p) 0 (lambda () (play "pistol.snd")))
+;(bind-key (char->integer #\o) 0 (lambda () "play oboe" (play "oboe.snd")))
+;(bind-key (char->integer #\p) 0 (lambda () "play pistol" (play "pistol.snd")))
 
 ;;; in this particular case, there's no need to hold the DAC open
 ;;;   but maybe this will come in handy someday
@@ -214,9 +193,9 @@
 
 (define (vector-synthesis driver files read-even-when-not-playing)
 
-  "(vector-synthesis driver files read-even-when-not-playing) uses 'driver', a \
-function of two args (the number of files, and the number of samples between calls) to decide which file to play.  If \
-'read-even-when-not-playing' is #t (default is #f), the input files are constantly \
+  "(vector-synthesis driver files read-even-when-not-playing) uses 'driver', a 
+function of two args (the number of files, and the number of samples between calls) to decide which file to play.  If 
+'read-even-when-not-playing' is #t (default is #f), the input files are constantly 
 read, even if not playing.  'files' is a list of files to be played."
   
   (let ((files-len (length files)))
@@ -227,7 +206,7 @@ read, even if not playing.  'files' is a list of files to be played."
 	       (data (make-sound-data chans bufsize))
 	       (readers (map make-file->frame files))
 	       (locs (make-vector files-len 0))
-	       (frames (make-vector files-len 0))
+	       (pframes (make-vector files-len 0))
 	       (current-file 0)
 	       (reading #t)
 	       (out-port (mus-audio-open-output mus-audio-default srate chans mus-lshort (* bufsize 2))))
@@ -236,7 +215,7 @@ read, even if not playing.  'files' is a list of files to be played."
 	      (begin
 		(do ((i 0 (1+ i)))
 		    ((= i files-len))
-		  (vector-set! frames i (mus-sound-frames (list-ref files i))))
+		  (vector-set! pframes i (mus-sound-frames (list-ref files i))))
 		(catch #t
 		       (lambda ()
 			 (while reading
@@ -295,7 +274,7 @@ read, even if not playing.  'files' is a list of files to be played."
 							       (lambda (f)
 								 (if (= f files-len)
 								     #f
-								     (or (< (vector-ref locs f) (vector-ref frames f))
+								     (or (< (vector-ref locs f) (vector-ref pframes f))
 									 (any-data-left (1+ f)))))))
 						       (any-data-left 0)))))))
 		       (lambda args (begin (snd-print (format #f "error ~A" args)) (car args))))
@@ -320,7 +299,7 @@ read, even if not playing.  'files' is a list of files to be played."
 
 (define play-with-amps
   (lambda (sound . amps)
-    "(play-with-amps snd &rest amps) plays snd with each channel scaled by the corresponding \
+    "(play-with-amps snd #:rest amps) plays snd with each channel scaled by the corresponding 
 amp: (play-with-amps 0 1.0 0.5) plays channel 2 of stereo sound at half amplitude"
     (let ((chans (chans sound)))
       (do ((chan 0 (1+ chan)))
@@ -330,3 +309,48 @@ amp: (play-with-amps 0 1.0 0.5) plays channel 2 of stereo sound at half amplitud
           (add-player player)))
       (start-playing chans (srate sound)))))
 
+
+;;; play-sine and play-sines
+
+(define* (play-sine freq amp)
+  "(play-sine freq amp) plays a 1 second sinewave at freq and amp"
+  (let* ((audio-info (open-play-output 1 22050 #f 256))
+	 (audio-fd (car audio-info))
+	 (outchans (cadr audio-info))
+	 (pframes (caddr audio-info)))
+    (if (not (= audio-fd -1))
+	(let ((len 22050)
+	      (osc (make-oscil freq))
+	      (data (make-sound-data outchans pframes)))
+	  (do ((beg 0 (+ beg pframes)))
+	      ((or (c-g?)                   ; C-g to stop in mid-stream
+		   (> beg len)))
+	    (do ((i 0 (1+ i)))
+		((= i pframes))
+	      (sound-data-set! data 0 i (* amp (oscil osc))))
+	    (mus-audio-write audio-fd data pframes))
+	  (mus-audio-close audio-fd))
+	#f)))
+
+(define* (play-sines freqs-and-amps)
+  "(play-sines freqs-and-amps) produces a tone given its spectrum: (play-sines '((440 .4) (660 .3)))"
+  (let* ((audio-info (open-play-output 1 22050 #f 256))
+	 (audio-fd (car audio-info))
+	 (outchans (cadr audio-info))
+	 (pframes (caddr audio-info)))
+    (if (not (= audio-fd -1))
+	(let ((len 22050)
+	      (oscs (map make-oscil (map car freqs-and-amps)))
+	      (amps (map cadr freqs-and-amps))
+	      (data (make-sound-data outchans pframes)))
+	  (do ((beg 0 (+ beg pframes)))
+	      ((or (c-g?)                   ; C-g to stop in mid-stream
+		   (> beg len)))
+	    (do ((i 0 (1+ i)))
+		((= i pframes))
+	      (sound-data-set! data 0 i (apply + (map (lambda (o a) (* a (oscil o))) oscs amps))))
+	    (mus-audio-write audio-fd data pframes))
+	  (mus-audio-close audio-fd))
+	#f)))
+
+;(play-sines '((425 .05) (450 .01) (470 .01) (546 .02) (667 .01) (789 .034) (910 .032)))
