@@ -8,6 +8,20 @@ int snd_round(double x) /* needs to be double here (not Float) for x axis calcs
   return(i);
 }
 
+off_t snd_round_off_t(double x)
+{
+  off_t i;
+  i = (off_t)x;
+  if ((x - i) > 0.5) return(i + 1);
+  return(i);
+}
+
+off_t snd_abs_off_t(off_t val)
+{
+  /* div is also limited to int */
+  return((val < 0) ? -val : val);
+}
+
 #define POW2_SIZE 31
 static int pow2s[POW2_SIZE] = {1, 2, 4, 8, 16, 32, 64, 128, 256, 
 			       512, 1024, 2048, 4096, 8192, 16384, 32768, 65536, 
@@ -15,9 +29,7 @@ static int pow2s[POW2_SIZE] = {1, 2, 4, 8, 16, 32, 64, 128, 256,
 			       33554432, 67108864, 134217728, 268435456, 536870912, 1073741824};
 int snd_ipow2(int n)
 {
-  if (n < POW2_SIZE)
-    return(pow2s[n]);
-  return((int)pow(2.0, n));
+  return(pow2s[n]);
 }
 
 int snd_2pow2(int n)
@@ -30,10 +42,14 @@ int snd_2pow2(int n)
   return(0);
 }
 
+Float in_dB(Float min_dB, Float lin_dB, Float py)
+{
+  return((py <= lin_dB) ? min_dB : (20.0 * (log10(py))));
+}
 
 char *copy_string(const char *str)
 {
-#if DEBUG_MEMORY || (!HAVE_STRDUP)
+#if DEBUG_MEMORY
   char *newstr = NULL;
   if (str)
     {
@@ -48,14 +64,49 @@ char *copy_string(const char *str)
 #endif
 }
 
-int snd_strlen(char *str)
+int snd_strlen(const char *str)
 {
   /* strlen(NULL) -> seg fault! */
   if ((str) && (*str)) return(strlen(str));
   return(0);
 }
 
-char *filename_without_home_directory(char *name)
+char *snd_strcat(char *errmsg, const char *str, int *size)
+{
+  int new_len, err_size;
+  new_len = (snd_strlen(str) + snd_strlen(errmsg));
+  err_size = size[0];
+  if (new_len >= err_size)
+    {
+      if ((err_size * 2) > new_len)
+	err_size = err_size * 2;
+      else err_size = new_len;
+      errmsg = (char *)REALLOC(errmsg, err_size * sizeof(char));
+      size[0] = err_size;
+    }
+  strcat(errmsg, str);
+  return(errmsg);
+}
+
+char *string_to_colon(char *val)
+{
+  char *up_to_colon;
+  int i, len;
+  up_to_colon = (char *)CALLOC(strlen(val) + 1, sizeof(char));
+  len = strlen(val);
+  for (i = 0; i < len; i++)
+    {
+      if ((val[i] == ':') || (val[i] == ' '))
+	{
+	  up_to_colon[i] = 0;
+	  return(up_to_colon);
+	}
+      up_to_colon[i] = val[i];
+    }
+  return(up_to_colon);
+}
+
+char *filename_without_home_directory(const char *name)
 {
   /* since we don't want to mess with freeing these guys, I'll just return a pointer into the name */
   int i, len, last_slash;
@@ -82,6 +133,30 @@ char *just_filename(char *name)
   return(nodir);
 }
 
+#if HAVE_LANGINFO_DECIMAL_POINT || HAVE_LANGINFO_RADIXCHAR
+#if NL_TYPES_H_DEFINES_MALLOC
+  /* SGI complication */
+  #undef MALLOC
+  #include <langinfo.h>
+  #undef MALLOC
+  #define MALLOC malloc
+#else
+  #include <langinfo.h>
+#endif
+#endif
+
+static char decimal_pt;
+static char local_decimal_point(void)
+{
+#if HAVE_LANGINFO_DECIMAL_POINT
+  return(nl_langinfo(DECIMAL_POINT)[0]);
+#endif
+#if HAVE_LANGINFO_RADIXCHAR
+  return(nl_langinfo(RADIXCHAR)[0]);
+#endif
+  return('.');
+}
+
 static char prtbuf[256];
 
 char *prettyf(Float num, int tens)
@@ -102,20 +177,20 @@ char *prettyf(Float num, int tens)
   if ((num - fullf) == 0.0) 
     {
       if (num < 100.0)
-	sprintf(prtbuf, "%d%c0", fullf, STR_decimal);
+	sprintf(prtbuf, "%d%c0", fullf, decimal_pt);
       else sprintf(prtbuf, "%d", fullf);
       return(copy_string(prtbuf));
     }
   if (num > 1000)
     {
-      sprintf(prtbuf, "%d%c%d", fullf, STR_decimal, (int)((num - fullf) * pow(10.0, tens)));
+      sprintf(prtbuf, "%d%c%d", fullf, decimal_pt, (int)((num - fullf) * pow(10.0, tens)));
       return(copy_string(prtbuf));
     }
   fullf = (int)(num * pow(10.0, tens + 1) + rounder);
   if (fullf == 0) 
     {
       /* will be freed later, so can't return a constant */
-      newval=(char *)MALLOC(2 * sizeof(char));
+      newval = (char *)MALLOC(2 * sizeof(char));
       newval[0] = '0';
       newval[1] = '\0';
       return(newval);
@@ -136,18 +211,18 @@ char *prettyf(Float num, int tens)
     {
       for (i = 0; i < len - tens - 1; i++) 
 	{
-	  (*sn) =(*sp); 
+	  (*sn) = (*sp); 
 	  sn++; 
 	  sp++;
 	}
-      (*sn) = STR_decimal; 
+      (*sn) = decimal_pt; 
       sn++;
     }
   else
     {
       (*sn) = '0';
       sn++;
-      (*sn) = STR_decimal;
+      (*sn) = decimal_pt;
       sn++;
       lim = abs(len - tens - 1);
       for (i = 0; i < lim; i++) 
@@ -171,27 +246,6 @@ char *prettyf(Float num, int tens)
   return(newval);
 }
 
-void fill_number(char *fs, char *ps)
-{
-  int i, j;
-  j = snd_strlen(fs);
-  if (j > 4) j = 4;
-  if (j < 4) 
-    {
-      ps[4] = '\0'; 
-      ps[3] = '0'; 
-      ps[2] = '0'; 
-      ps[1] = STR_decimal;
-    }
-  if ((*fs) == STR_decimal) 
-    {
-      *ps++ = '0'; 
-      if (j == 4) j = 3;
-    }
-  for (i = 0; i < j; i++) 
-    (*ps++) = (*fs++);
-}
-
 static char *get_tmpdir(void)
 {
   char *tmpdir = NULL;
@@ -210,7 +264,7 @@ static char *get_tmpdir(void)
 }
 
 static int sect_ctr = 0;
-char *shorter_tempnam(char *udir, char *prefix)
+char *shorter_tempnam(const char *udir, const char *prefix)
 {
   /* tempnam turns out names that are inconveniently long (in this case the filename is user-visible) */
   char *str, *tmpdir = NULL;
@@ -220,10 +274,12 @@ char *shorter_tempnam(char *udir, char *prefix)
   else tmpdir = copy_string(udir);
   mus_snprintf(str, PRINT_BUFFER_SIZE, "%s/%s%d_%d.snd", tmpdir, (prefix) ? prefix : "snd_", (int)getpid(), sect_ctr++);
   if (tmpdir) FREE(tmpdir);
-  return(str);
+  tmpdir = copy_string(str);
+  FREE(str);
+  return(tmpdir);
 }
 
-char *snd_tempnam(snd_state *ss)
+char *snd_tempnam(void)
 {
   /* problem here is that NULL passed back from Guile becomes "" which is not NULL from tempnam's point of view */
   char *udir;
@@ -240,7 +296,22 @@ void snd_exit(int val)
 #endif
 }
 
-char *kmg (int num)
+#if __GNUC__
+#if WITH_EFENCE && (!DEBUG_MEMORY)
+  #warning compile switches are inconsistent (with_efence needs debug_memory)
+#endif
+#endif
+
+#ifdef DEBUG_MEMORY
+
+/* mtrace-style malloc hooks are not very useful here since I don't care
+ * about X allocations (of which there are millions), and I need readable
+ * backtrace info for leaks.  Doing it by hand makes it easy to sort
+ * output and whatnot. All of Sndlib and Snd use the macros CALLOC,
+ * MALLOC, REALLOC, and FREE.
+ */
+
+static char *kmg (int num)
 {
   /* return number 0..1024, then in terms of K, M, G */
   char *str;
@@ -259,85 +330,32 @@ char *kmg (int num)
   return(str);
 }
 
-#if HAVE_EXECINFO_H
-/* from glibc/debug/backtrace-tst.c, need to build Snd with -rdynamic */
-    
-#include <execinfo.h>
-#include <inttypes.h>
-XEN show_stack(void)
-{
-  void *ba[20];
-  int n = backtrace (ba, sizeof (ba) / sizeof (ba[0]));
-  if (n != 0)
-    {
-      char **names = backtrace_symbols (ba, n);
-      if (names != NULL)
-	{
-	  int i;
-	  printf ("called from %s\n", names[0]);
-	  for (i = 1; i < n; ++i)
-	    printf ("            %s\n", names[i]);
-	  free (names);
-	}
-    }
-  return(XEN_FALSE);
-}
-#else
-XEN show_stack(void) {return(XEN_FALSE);}
-#endif
-
-#ifdef DEBUG_MEMORY
-
-/* mtrace-style malloc hooks are not very useful here since I don't care
- * about X allocations (of which there are millions), and I need readable
- * backtrace info for leaks.  Doing it by hand makes it easy to sort
- * output and whatnot. All of Sndlib and Snd use the macros CALLOC,
- * MALLOC, REALLOC, and FREE.
- */
-
-static int mem_size = 0;
+static int mem_size = 0, mem_top = -1;
 static int *pointers = NULL, *sizes = NULL, *locations = NULL;
 static char **functions = NULL, **files = NULL;
 static int *lines = NULL;
 static int mem_location = -1;
 static int mem_locations = 0;
-static int last_mem_location = -1;
-static char ***stacks = NULL;
-static int *stack_sizes = NULL;
+static char **stacks = NULL;
 
-static char **encloser = NULL;
-static int stack_size = 0;
-void set_encloser(const char *name) 
+static char *encloser = NULL;
+void set_encloser(char *name) 
 {
-  /* for exposing call chains */
-#if HAVE_EXECINFO_H
-  void *ba[6];
-  int n;
-  if (name)
-    {
-      n = backtrace(ba, sizeof (ba) / sizeof (ba[0]));
-      if (n != 0)
-	{
-	  encloser = backtrace_symbols(ba, n);
-	  stack_size = n;
-	}
-      else encloser = NULL;
-    }
+  if (name) encloser = name;
   else encloser = NULL;
-#endif
 }
 
+static int *lines_hit = NULL;
+
 static int find_mem_location(const char *func, const char *file, int line)
 {
   int i;
-  if ((last_mem_location >= 0) &&
-      (line == lines[last_mem_location]) &&
-      (strcmp(func, functions[last_mem_location]) == 0) &&
-      (strcmp(file, files[last_mem_location]) == 0))
-    return(last_mem_location);
+  if ((lines_hit) && 
+      (lines_hit[line] > 0) &&
+      (strcmp(file, files[lines_hit[line]]) == 0))
+    return(lines_hit[line]);
   for (i = 0; i <= mem_location; i++)
     if ((line == lines[i]) &&
-	(strcmp(func, functions[i]) == 0) &&
 	(strcmp(file, files[i]) == 0))
       return(i);
   mem_location++;
@@ -345,16 +363,19 @@ static int find_mem_location(const char *func, const char *file, int line)
     {
       if (mem_locations == 0)
 	{
-	  functions = (char **)calloc(1024, sizeof(char *));
-	  files = (char **)calloc(1024, sizeof(char *));
-	  lines = (int *)calloc(1024, sizeof(int));
-	  mem_locations = 1024;
+	  functions = (char **)calloc(2048, sizeof(char *));
+	  files = (char **)calloc(2048, sizeof(char *));
+	  lines = (int *)calloc(2048, sizeof(int));
+
+	  lines_hit = (int *)calloc(65536, sizeof(int));
+
+	  mem_locations = 2048;
 	}
       else
 	{
-	  functions = (char **)realloc(functions, (mem_locations + 1024)*sizeof(char *));
-	  files = (char **)realloc(files, (mem_locations + 1024)*sizeof(char *));
-	  lines = (int *)realloc(lines, (mem_location + 1024)*sizeof(int));
+	  functions = (char **)realloc(functions, (mem_locations + 1024) * sizeof(char *));
+	  files = (char **)realloc(files, (mem_locations + 1024) * sizeof(char *));
+	  lines = (int *)realloc(lines, (mem_location + 1024) * sizeof(int));
 	  for (i = 0; i < 1024; i++) 
 	    {
 	      functions[i + mem_locations] = NULL;
@@ -365,12 +386,18 @@ static int find_mem_location(const char *func, const char *file, int line)
 	}
     }
   /* NOT copy_string HERE!! */
-  last_mem_location = mem_location;
   functions[mem_location] = (char *)calloc(strlen(func) + 1, sizeof(char));
   strcpy(functions[mem_location], func);
   files[mem_location] = (char *)calloc(strlen(file) + 1, sizeof(char));
   strcpy(files[mem_location], file);
   lines[mem_location] = line;
+
+  if (line < 65536)
+    {
+      if (lines_hit[line] == 0) 
+	lines_hit[line] = mem_location;
+      else lines_hit[line] = -1;
+    }
   return(mem_location);
 }
 
@@ -380,24 +407,23 @@ static int last_remembered = -1, last_remembered_ptr = -1;
 static void forget_pointer(void *ptr, const char *func, const char *file, int line)
 {
   int i;
-  /* fprintf(stderr,"forget %p (%s %s %d)\n",ptr,func,file,line); */
   if (ptr == NULL) {fprintf(stderr, "attempt to free NULL"); mem_report(); abort();}
   if (last_remembered_ptr == (int)ptr)
     {
       pointers[last_remembered] = 0;
       stacks[last_remembered] = NULL;
-      stack_sizes[last_remembered] = 0;
       last_forgotten = last_remembered;
       last_remembered_ptr = -1;
       return;
     }
-  for (i = 0; i < mem_size; i++)
+  for (i = mem_top; i >= 0; i--)
     if (pointers[i] == (int)ptr)
       {
 	pointers[i] = 0;
 	stacks[i] = NULL;
-	stack_sizes[i] = 0;
 	last_forgotten = i;
+	if (i == mem_top)
+	  while ((mem_top >= 0) && (pointers[mem_top] == 0)) mem_top--;
 	return;
       }
   fprintf(stderr, "forget %p ", ptr); mem_report(); abort();
@@ -406,7 +432,6 @@ static void forget_pointer(void *ptr, const char *func, const char *file, int li
 static void remember_pointer(void *ptr, size_t len, const char *func, const char *file, int line)
 {
   int i, loc = 0;
-  /* fprintf(stderr,"remember %p (%s %s %d)\n",ptr,func,file,line); */
   if (last_forgotten == -1)
     {
       if (mem_size == 0)
@@ -415,8 +440,7 @@ static void remember_pointer(void *ptr, size_t len, const char *func, const char
 	  pointers = (int *)calloc(mem_size, sizeof(int));
 	  sizes = (int *)calloc(mem_size, sizeof(int));
 	  locations = (int *)calloc(mem_size, sizeof(int));
-	  stacks = (char ***)calloc(mem_size, sizeof(char **));
-	  stack_sizes = (int *)calloc(mem_size, sizeof(int));
+	  stacks = (char **)calloc(mem_size, sizeof(char *));
 	  loc = 0;
 	  goto GOT_ONE;
 	}
@@ -438,15 +462,13 @@ static void remember_pointer(void *ptr, size_t len, const char *func, const char
       pointers = (int *)realloc(pointers, mem_size * sizeof(int));
       sizes = (int *)realloc(sizes, mem_size * sizeof(int));
       locations = (int *)realloc(locations, mem_size * sizeof(int));
-      stacks = (char ***)realloc(stacks, mem_size * sizeof(char **));
-      stack_sizes = (int *)realloc(stack_sizes, mem_size * sizeof(int));
+      stacks = (char **)realloc(stacks, mem_size * sizeof(char *));
       for (i = loc; i < mem_size; i++)
 	{
 	  pointers[i] = 0;
 	  sizes[i] = 0;
 	  locations[i] = 0;
 	  stacks[i] = NULL;
-	  stack_sizes[i] = 0;
 	}
     }
   else
@@ -458,26 +480,35 @@ static void remember_pointer(void *ptr, size_t len, const char *func, const char
   pointers[loc] = (int)ptr;
   sizes[loc] = (int)len;
   locations[loc] = find_mem_location(func, file, line);
-  if (encloser) 
-    {
-      stacks[loc] = encloser;
-      stack_sizes[loc] = stack_size;
-    }
+  if (encloser) stacks[loc] = encloser;
   last_remembered = loc;
   last_remembered_ptr = (int)ptr;
+  if (mem_top < loc) mem_top = loc;
 }
 
 #define MAX_MALLOC (1 << 28)
 
+#if WITH_EFENCE
+void *ef_calloc(size_t a, size_t b);
+void *ef_malloc(size_t a);
+void *ef_free(void *a);
+void *ef_realloc(void *a, size_t b);
+#endif
+
 void *mem_calloc(size_t len, size_t size, const char *func, const char *file, int line)
 {
   void *ptr;
   if ((len == 0) || (len > MAX_MALLOC))
     {
-      fprintf(stderr, "%s:%s[%d] attempt to calloc %d bytes", func, file, line, len*size);
+      fprintf(stderr, "%s:%s[%d] attempt to calloc %d bytes", func, file, line, len * size);
       mem_report(); abort();
     }
+#if WITH_EFENCE
+  ptr = ef_calloc(len, size);
+#else
   ptr = calloc(len, size);
+#endif
+  if (ptr == NULL) {fprintf(stderr,"calloc->null"); abort();}
   remember_pointer(ptr, len * size, func, file, line);
   return(ptr);
 }
@@ -490,15 +521,25 @@ void *mem_malloc(size_t len, const char *func, const char *file, int line)
       fprintf(stderr, "%s:%s[%d] attempt to malloc %d bytes", func, file, line, len);
       mem_report(); abort();
     }
+#if WITH_EFENCE
+  ptr = ef_malloc(len);
+#else
   ptr = malloc(len);
+#endif
+  if (ptr == NULL) {fprintf(stderr,"malloc->null"); abort();}
   remember_pointer(ptr, len, func, file, line);
   return(ptr);
 }
 
 void mem_free(void *ptr, const char *func, const char *file, int line)
 {
+  /* fprintf(stderr,"free %s %s[%d]: %p\n", func, file, line, ptr); */
   forget_pointer(ptr, func, file, line);
+#if WITH_EFENCE
+  ef_free(ptr);
+#else
   free(ptr);
+#endif
 }
 
 void *mem_realloc(void *ptr, size_t size, const char *func, const char *file, int line)
@@ -510,12 +551,17 @@ void *mem_realloc(void *ptr, size_t size, const char *func, const char *file, in
       mem_report(); abort();
     }
   forget_pointer(ptr, func, file, line);
+#if WITH_EFENCE
+  new_ptr = ef_realloc(ptr, size);
+#else
   new_ptr = realloc(ptr, size);
+#endif
+  if (new_ptr == NULL) {fprintf(stderr,"realloc->null"); abort();}
   remember_pointer(new_ptr, size, func, file, line);
   return(new_ptr);
 }
 
-static char *mem_stats(snd_state *ss, int ub)
+static char *mem_stats(int ub)
 {
   int i, ptrs = 0, sum = 0, snds = 0, chns = 0;
   snd_info *sp;
@@ -528,11 +574,14 @@ static char *mem_stats(snd_state *ss, int ub)
       }
   result = (char *)calloc(PRINT_BUFFER_SIZE, sizeof(char));
   for (i = 0; i < ss->max_sounds; i++)
-    if ((sp=((snd_info *)(ss->sounds[i]))))
-      {
-	snds++;
-	chns += sp->allocated_chans;
-      }
+    {
+      sp = ss->sounds[i];
+      if (sp)
+	{
+	  snds++;
+	  chns += sp->allocated_chans;
+	}
+    }
   mus_snprintf(result, PRINT_BUFFER_SIZE, "snd mem: %s (%s ptrs), %d sounds, %d chans (%s)\n",
 	  ksum = kmg(sum),
 	  kptrs = kmg(ptrs),
@@ -544,21 +593,146 @@ static char *mem_stats(snd_state *ss, int ub)
   return(result);
 }
 
+
+/* file bookkeepping */
+static char **open_files = NULL, **file_funcs = NULL, **file_files = NULL;
+static int *file_lines = NULL, *file_fds = NULL;
+static FILE **file_fls = NULL;
+static int file_size = 0;
+
+static void io_add(const char *pathname, const char *func, const char *file, int line, int fd, FILE *fl)
+{
+  int i, loc = -1;
+  if (file_size == 0)
+    {
+      file_size = 128;
+      open_files = (char **)calloc(file_size, sizeof(char *));
+      file_funcs = (char **)calloc(file_size, sizeof(char *));
+      file_files = (char **)calloc(file_size, sizeof(char *));
+      file_lines = (int *)calloc(file_size, sizeof(int));
+      file_fds = (int *)calloc(file_size, sizeof(int));
+      for (i = 0; i < file_size; i++) file_fds[i] = -1;
+      file_fls = (FILE **)calloc(file_size, sizeof(FILE *));
+      open_files[0] = copy_string(pathname);
+      file_funcs[0] = copy_string(func);
+      file_files[0] = copy_string(file);
+      file_lines[0] = line;
+      file_fds[0] = fd;
+      file_fls[0] = fl;
+    }
+  else
+    {
+      for (i = 0; i < file_size; i++)
+	if (open_files[i] == NULL)
+	  {
+	    loc = i;
+	    break;
+	  }
+      if (loc == -1)
+	{
+	  loc = file_size;
+	  file_size += 128;
+	  open_files = (char **)realloc(open_files, file_size * sizeof(char *));
+	  file_funcs = (char **)realloc(file_funcs, file_size * sizeof(char *));
+	  file_files = (char **)realloc(file_files, file_size * sizeof(char *));
+	  file_lines = (int *)realloc(file_lines, file_size * sizeof(int));
+	  file_fds = (int *)realloc(file_fds, file_size * sizeof(int));
+	  for (i = loc; i < file_size; i++) 
+	    {
+	      file_fds[i] = -1;
+	      open_files[i] = NULL;
+	      file_files[i] = NULL;
+	      file_funcs[i] = NULL;
+	    }
+	  file_fls = (FILE **)realloc(file_fls, file_size * sizeof(FILE *));
+	}
+      open_files[loc] = copy_string(pathname);
+      file_funcs[loc] = copy_string(func);
+      file_files[loc] = copy_string(file);
+      file_lines[loc] = line;
+      file_fds[loc] = fd;
+      file_fls[loc] = fl;
+    }
+}
+
+static void io_subtract(int fd, FILE *fl, const char *func, const char *file, int line)
+{
+  int i;
+  for (i = 0; i < file_size; i++)
+    if (((fd >= 0) && (file_fds[i] == fd)) ||
+	((fl) && (file_fls[i] == fl)))
+      {
+	if (((file_fls[i]) && (fd >= 0)) ||
+	    ((fl) && (file_fds[i] >= 0)))
+	  fprintf(stderr,"%s: %s[%d] (%s) %d %p?\n", file_files[i], file_funcs[i], file_lines[i], open_files[i], file_fds[i], file_fls[i]);
+	FREE(file_files[i]); file_files[i] = NULL;
+	FREE(file_funcs[i]); file_funcs[i] = NULL;
+	FREE(open_files[i]); open_files[i] = NULL;
+	file_fds[i] = -1;
+	file_fls[i] = NULL;
+	return;
+      }
+  fprintf(stderr, "%s: %s[%d] can't find %d %p?\n", file, func, line, fd, fl);
+}
+
+int io_open(const char *pathname, int flags, mode_t mode, const char *func, const char *file, int line)
+{
+  int fd;
+  fd = open(pathname, flags, mode);
+  if (fd != -1) io_add(pathname, func, file, line, fd, NULL);
+  return(fd);
+}
+
+int io_creat(const char *pathname, mode_t mode, const char *func, const char *file, int line)
+{
+  int fd;
+  fd = creat(pathname, mode);
+  if (fd != -1) io_add(pathname, func, file, line, fd, NULL);
+  return(fd);
+}
+
+int io_close(int fd, const char *func, const char *file, int line)
+{
+  io_subtract(fd, NULL, func, file, line);
+  return(close(fd));
+}
+
+FILE *io_fopen(const char *path, const char *mode, const char *func, const char *file, int line)
+{
+  FILE *fp;
+  fp = fopen(path, mode);
+  if (fp) io_add(path, func, file, line, -1, fp);
+  return(fp);
+}
+
+int io_fclose(FILE *stream, const char *func, const char *file, int line)
+{
+  io_subtract(-1, stream, func, file, line);
+  return(fclose(stream));
+}
+
+void dump_protection(FILE *Fp);
+#if DEBUGGING
+  void report_dangling_readers(FILE *fp);
+  void report_dangling_mix_readers(FILE *fp);
+#endif
+
 void mem_report(void)
 {
-  int loc, i, j, k, sum, ptr = 0, have_stacks = 0;
+  int loc, i, j, sum, ptr = 0;
+  bool have_stacks = false;
   int *sums, *ptrs;
   FILE *Fp;
   time_t ts;
   char time_buf[TIME_STR_SIZE];
-
+  if (ss->search_tree)
+    ss->search_tree = free_ptree(ss->search_tree);
   for (i = 0; i < mem_size; i++)
     if (stacks[i])
       {
-	have_stacks = 1;
+	have_stacks = true;
 	break;
       }
-
   sums = (int *)calloc(mem_location + 1, sizeof(int));
   ptrs = (int *)calloc(mem_location + 1, sizeof(int));
   for (loc = 0; loc <= mem_location; loc++)
@@ -575,10 +749,16 @@ void mem_report(void)
       ptrs[loc] = ptr;
     }
   Fp = fopen("memlog", "w");
+  if (Fp == NULL) return;
 
   time(&ts);
   strftime(time_buf, TIME_STR_SIZE, STRFTIME_FORMAT, localtime(&ts));
-  fprintf(Fp, "memlog: %s: %s\n\n", time_buf, mem_stats(get_global_state(), 0));
+  {
+    char *str;
+    str = mem_stats(0);
+    fprintf(Fp, "memlog: %s: %s\n\n", time_buf, str);
+    free(str);
+  }
 
   for (i = 0; i <= mem_location; i++)
     {
@@ -591,17 +771,57 @@ void mem_report(void)
 	  }
       if (sum > 0)
 	{
-	  fprintf(Fp, "%s[%d]:%s:  %d (%d)\n", files[ptr], lines[ptr], functions[ptr], sums[ptr], ptrs[ptr]);
+	  fprintf(Fp, "%s[%d]:%s:  %d (%d)", files[ptr], lines[ptr], functions[ptr], sums[ptr], ptrs[ptr]);
+	  if (0)
+	    {
+	      int fd, k, line, bytes;
+	      bool happy = true;
+	      char buf[8192];
+	      fd = open(files[ptr], O_RDONLY, 0);
+	      line = 1;
+	      while (happy)
+		{
+		  bytes = read(fd, buf, 8192);
+		  if (bytes <= 0) 
+		    {
+		      fprintf(Fp, "where is %s[%d]?\n", files[ptr], lines[ptr]);
+		      happy = false;
+		      break;
+		    }
+		  for (k = 0; k < bytes; k++)
+		    {
+		      if (buf[k] == '\n')
+			{
+			  line++;
+			  if (line == lines[ptr]) fprintf(Fp, ":   ");
+			  if (line > lines[ptr]) 
+			    {
+			      fprintf(Fp, "\n");
+			      happy = false;
+			      break;
+			    }
+			}
+		      else
+			{
+			  if (line == lines[ptr]) fprintf(Fp, "%c", buf[k]);
+			}
+		    }
+		}
+	      close(fd);
+	    }
+	  else fprintf(Fp, "\n");
 	  sums[ptr] = 0;
 	  if (have_stacks)
+	    for (j = 0; j < mem_size; j++)
+	      if ((stacks[j]) && (locations[j] == ptr) && (pointers[j]))
+		fprintf(Fp, "    %s    %p\n", stacks[j], (void *)(pointers[j]));
+	  if ((strcmp("mus_format", functions[ptr]) == 0) ||
+	      (strcmp("copy_string", functions[ptr]) == 0))
 	    {
 	      for (j = 0; j < mem_size; j++)
-		if ((stacks[j]) && (locations[j] == ptr) && (pointers[j]))
-		  {
-		    fprintf(Fp, "stack: \n");
-		    for (k = 0; k < stack_sizes[j]; k++)
-		      fprintf(Fp, "    %s\n", stacks[j][k]);
-		  }
+		if ((locations[j] == ptr) && (pointers[j]))
+		  fprintf(Fp, "[%s] ", (char *)(pointers[j]));
+	      fprintf(Fp, "\n");
 	    }
 	}
     }
@@ -611,6 +831,18 @@ void mem_report(void)
   fprintf(Fp, "\n\nprevlist: %d %d\n", get_prevfile_end(), get_max_prevfile_end());
   fprintf(Fp, "\n\n");
   save_listener_text(Fp);
+  /* mus_sound_report_cache(Fp); */
+#if DEBUGGING
+  report_dangling_readers(Fp);
+  report_dangling_mix_readers(Fp);
+#endif
+
+  for (i = 0; i < file_size; i++)
+    if (open_files[i])
+      fprintf(Fp, "%s: %s[%d] (%s) %d %p?\n", file_files[i], file_funcs[i], file_lines[i], open_files[i], file_fds[i], file_fls[i]);
+  dump_protection(Fp);
+  free(sums);
+  free(ptrs);
   fclose(Fp);
 }
 
@@ -628,3 +860,34 @@ void stop_timing(void) {fprintf(stderr, "time: %d ",(int)((clock() - start) * 10
 
 #endif
 
+#if HAVE_GUILE
+#define S_file2string "file->string"
+static XEN g_file_to_string(XEN name)
+{ 
+  char *filename;
+  FILE *file;
+  int size;
+  char *content = NULL;
+  XEN val = XEN_FALSE;
+  XEN_ASSERT_TYPE(XEN_STRING_P(name), name, XEN_ONLY_ARG, S_file2string, "a string");
+  filename = XEN_TO_C_STRING(name);
+  if ((file = fopen(filename, "r")) == NULL) return(XEN_FALSE);
+  fseek(file, 0, SEEK_END);
+  size = ftell(file); /* safe because this is presumably not a huge file */
+  rewind(file);
+  content = (char *)CALLOC(size + 1, sizeof(char));
+  fread(content, 1, size, file);
+  fclose(file);
+  val = C_TO_XEN_STRING(content);
+  FREE(content);
+  return(val);
+}
+#endif
+
+void g_init_utils(void)
+{
+  decimal_pt = local_decimal_point();
+#if HAVE_GUILE
+  XEN_DEFINE_PROCEDURE(S_file2string, g_file_to_string, 1, 0, 0, "file contents as string");
+#endif
+}
