@@ -1,13 +1,10 @@
 /* sndplay plays sounds */
 
-#if defined(HAVE_CONFIG_H)
-  #include "config.h"
-#endif
-
+#include <config.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#if (defined(NEXT) || (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H))))
+#if (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H)))
   #include <libc.h>
 #else
   #if (!(defined(_MSC_VER))) && (!(defined(MPW_C)))
@@ -23,31 +20,29 @@
   #include <console.h>
 #endif
 
-#if defined(NEXT) 
-  #define BUFFER_SIZE 1024
+#if MAC_OSX
+  #define BUFFER_SIZE 256
 #else
-  #if defined(MAC_OSX)
-    #define BUFFER_SIZE 512
-  #else
-    #if defined(WIN32)
-      /* this setting from Scott Middleton (actually used 8096) */
-      #define BUFFER_SIZE 8192
-    #else
-      #define BUFFER_SIZE 4096
-    #endif
-  #endif
+#if WIN32
+  /* this setting from Scott Middleton (actually used 8096) */
+  #define BUFFER_SIZE 8192
+#else
+  #define BUFFER_SIZE 4096
+#endif
 #endif
 
 #if MAC_OSX
   #define OutSample float
+  #define MUS_CONVERT(samp) MUS_SAMPLE_TO_FLOAT(samp)
 #else
   #define OutSample short
+  #define MUS_CONVERT(samp) MUS_SAMPLE_TO_SHORT(samp)
 #endif
 
-#if defined(LINUX) || defined(__bsdi__)
-static char x_string[2] = {'x','\0'};
 static void set_buffers(char *bufs)
 {
+#if (HAVE_OSS || HAVE_ALSA)
+static char x_string[2] = {'x','\0'};
   char *arg;
   int a, b;
   arg = strtok(bufs, x_string);
@@ -55,8 +50,8 @@ static void set_buffers(char *bufs)
   arg = strtok(NULL, x_string);
   b = atoi(arg);
   mus_audio_set_oss_buffers(a, b);
-}
 #endif
+}
 
 /* special case multicard quad code split out for clarity (it could be folded into the main branches) */
 
@@ -65,58 +60,74 @@ void *mem_calloc(size_t len, size_t size, const char *func, const char *file, in
 void *mem_malloc(size_t len, const char *func, const char *file, int line) {return(malloc(len));}
 void mem_free(void *ptr, const char *func, const char *file, int line) {free(ptr);}
 void *mem_realloc(void *ptr, size_t size, const char *func, const char *file, int line) {return(realloc(ptr, size));}
+#include <fcntl.h>
+int io_open(const char *pathname, int flags, mode_t mode, const char *func, const char *file, int line) {return(open(pathname, flags, mode));}
+int io_creat(const char *pathname, mode_t mode, const char *func, const char *file, int line) {return(creat(pathname, mode));}
+int io_close(int fd, const char *func, const char *file, int line) {return(close(fd));}
+FILE *io_fopen(const char *path, const char *mode, const char *func, const char *file, int line) {return(fopen(path, mode));}
+int io_fclose(FILE *stream, const char *func, const char *file, int line) {return(fclose(stream));}
+char *copy_string(const char *str);
+char *copy_string(const char *str) {return(strdup(str));}
 #endif
 
 /* 22-Nov-00: moved alsa support to separate block */
+/* 8-Apr-04:  added start/end (seconds-based) args */
 
-#if (!HAVE_ALSA)
-int main(int argc, char *argv[])
+static int main_not_alsa(int argc, char *argv[])
 {
-  int fd, afd, i, j, n, k, chans, srate, frames, outbytes;
-  MUS_SAMPLE_TYPE **bufs;
+  int fd, afd, i, j, n, k, chans, srate;
+  off_t frames, m;
+  mus_sample_t **bufs;
   OutSample *obuf;
   float val[1];
   int use_multi_card_code = 0;
-  int afd0, afd1, buffer_size, curframes, sample_size, out_chans;
-  MUS_SAMPLE_TYPE **qbufs;
+  int afd0, afd1, buffer_size = BUFFER_SIZE, curframes, sample_size, out_chans, outbytes;
+  mus_sample_t **qbufs;
   short *obuf0, *obuf1;
   char *name = NULL;
-#if MACOS
-  argc = ccommand(&argv);
-#endif
-  if (argc == 1) 
-    {
-      printf("usage: sndplay file\n"); 
-      exit(0);
-    }
-  mus_sound_initialize();
-#if defined(LINUX) || defined(__bsdi__)
-  /* for clisp-based clm use, we need a couple added switches
-   * -describe => call mus_audio_describe and exit
-   * -buffers axb => set OSS fragment numbers 
-   */
-  for (i = 1; i < argc; )
+  off_t start = 0, end = 0;
+  double begin_time = 0.0, end_time = 0.0;
+
+  for (i = 1; i < argc; i++)
     {
       if (strcmp(argv[i], "-describe") == 0) 
 	{
 	  mus_audio_describe(); 
 	  exit(0);
 	}
-      else 
-	if (strcmp(argv[i], "-buffers") == 0) 
-	  {
-	    set_buffers(argv[i + 1]); 
-	    i++;
-	  }
-      else name = argv[i];
-      i++;
-    }
-#else
-  name = argv[1];
-#endif
+      else
+	{
+	  if (strcmp(argv[i], "-buffers") == 0) 
+	    {
+	      set_buffers(argv[i + 1]); 
+	      i++;
+	    }
+	  else
+	    {
+	      if (strcmp(argv[i], "-bufsize") == 0) 
+		{
+		  buffer_size = atoi(argv[i + 1]);
+		  i++;
+		}
+	      else
+		{
+		  if (strcmp(argv[i], "-start") == 0) 
+		    {
+		      begin_time = atof(argv[i + 1]);
+		      i++;
+		    }
+		  else
+		    {
+		      if (strcmp(argv[i], "-end") == 0) 
+			{
+			  end_time = atof(argv[i + 1]);
+			  i++;
+			}
+		      else name = argv[i];
+		    }}}}}
   if (name == NULL) 
     {
-      printf("usage: sndplay file\n"); 
+      printf("usage: sndplay file [-start 1.0] [-end 1.0] [-bufsize 256] [-buffers 2x12] [-describe]\n"); 
       exit(0);
     }
 
@@ -158,87 +169,36 @@ int main(int argc, char *argv[])
 		}
 	    }
 	}
-#if MAC_OSX
-      /* Mac OSX built-in device has only one "format": 44100 stereo bfloat 4096 bytes */
-      out_chans = 2;
-#else
       out_chans = chans;
-#endif
       srate = mus_sound_srate(name);
-      frames = mus_sound_samples(name) / chans;
-      sample_size = mus_data_format_to_bytes_per_sample(MUS_COMPATIBLE_FORMAT);
+      frames = mus_sound_frames(name);
+      sample_size = mus_bytes_per_sample(MUS_COMPATIBLE_FORMAT);
+      start = (off_t)(begin_time * srate);
+      if (start > 0)
+	mus_sound_seek_frame(fd, start);
+      if (end_time > 0.0)
+	end = (off_t)(end_time * srate);
+      else end = frames;
+      if ((end - start) < frames)
+	frames = end - start;
       if (!use_multi_card_code)
 	{
-	  outbytes = BUFFER_SIZE * out_chans * sample_size;
-	  bufs = (MUS_SAMPLE_TYPE **)CALLOC(chans, sizeof(MUS_SAMPLE_TYPE *));
-	  for (i = 0; i < chans; i++) bufs[i] = (MUS_SAMPLE_TYPE *)CALLOC(BUFFER_SIZE, sizeof(MUS_SAMPLE_TYPE));
-	  obuf = (OutSample *)CALLOC(BUFFER_SIZE * out_chans, sizeof(OutSample));
-#if MAC_OSX
-	  buffer_size = 512;
-	  if (srate == 22050) buffer_size = 256;
-#else
-	  buffer_size = BUFFER_SIZE;
-#endif
-	  for (i = 0; i < frames; i += buffer_size)
+	  bufs = (mus_sample_t **)CALLOC(chans, sizeof(mus_sample_t *));
+	  for (i = 0; i < chans; i++) bufs[i] = (mus_sample_t *)CALLOC(buffer_size, sizeof(mus_sample_t));
+	  obuf = (OutSample *)CALLOC(buffer_size * out_chans, sizeof(OutSample));
+	  outbytes = buffer_size * out_chans * sample_size;
+	  for (m = 0; m < frames; m += buffer_size)
 	    {
-	      if ((i + buffer_size) <= frames)
+	      if ((m + buffer_size) <= frames)
 		curframes = buffer_size;
-	      else curframes = frames - i;
+	      else curframes = frames - m;
 	      mus_sound_read(fd, 0, curframes - 1, chans, bufs); 
 	      /* some systems are happier if we read the file before opening the dac */
 	      /* at this point the data is in separate arrays of ints */
-#if MAC_OSX
-	      if (chans == 1)
-		{
-		  if (srate == 44100)
-		    {
-		      for (k = 0, n = 0; k < curframes; k++, n += 2) 
-			{
-			  obuf[n] = MUS_SAMPLE_TO_FLOAT(bufs[0][k]);
-			  obuf[n + 1] = 0.0;
-			}
-		    }
-		  else
-		    {
-		      for (k = 0, n = 0; k < curframes; k++, n += 4) 
-			{
-			  obuf[n] = MUS_SAMPLE_TO_FLOAT(bufs[0][k]);
-			  obuf[n + 1] = 0.0;
-			  obuf[n + 2] = obuf[n];
-			  obuf[n + 3] = 0.0;
-			}
-		    }
-		}
-	      else
-		{
-		  if (chans == 2)
-		    {
-		      if (srate == 44100)
-			{
-			  for (k = 0, n = 0; k < curframes; k++, n += 2) 
-			    {
-			      obuf[n] = MUS_SAMPLE_TO_FLOAT(bufs[0][k]); 
-			      obuf[n + 1] = MUS_SAMPLE_TO_FLOAT(bufs[1][k]);
-			    }
-			}
-		      else
-			{
-			  for (k = 0, n = 0; k < curframes; k++, n += 4) 
-			    {
-			      obuf[n] = MUS_SAMPLE_TO_FLOAT(bufs[0][k]); 
-			      obuf[n + 1] = MUS_SAMPLE_TO_FLOAT(bufs[1][k]);
-			      obuf[n + 2] = obuf[n];
-			      obuf[n + 3] = obuf[n + 1];
-			    }
-			}
-		    }
-		}
-#else
-	      /* not OSX */
 	      if (chans == 1)
 		{
 		  for (k = 0; k < curframes; k++) 
-		    obuf[k] = MUS_SAMPLE_TO_SHORT(bufs[0][k]);
+		    obuf[k] = MUS_CONVERT(bufs[0][k]);
 		}
 	      else
 		{
@@ -246,8 +206,8 @@ int main(int argc, char *argv[])
 		    {
 		      for (k = 0, n = 0; k < curframes; k++, n += 2) 
 			{
-			  obuf[n] = MUS_SAMPLE_TO_SHORT(bufs[0][k]); 
-			  obuf[n + 1] = MUS_SAMPLE_TO_SHORT(bufs[1][k]);
+			  obuf[n] = MUS_CONVERT(bufs[0][k]); 
+			  obuf[n + 1] = MUS_CONVERT(bufs[1][k]);
 			}
 		    }
 		  else
@@ -255,11 +215,10 @@ int main(int argc, char *argv[])
 		      for (k = 0, j = 0; k < curframes; k++, j += chans)
 			{
 			  for (n = 0; n < chans; n++) 
-			    obuf[j + n] = MUS_SAMPLE_TO_SHORT(bufs[n][k]);
+			    obuf[j + n] = MUS_CONVERT(bufs[n][k]);
 			}
 		    }
 		}
-#endif
 	      if (afd == -1)
 		{
 #if defined(LINUX) && defined(PPC)
@@ -269,11 +228,7 @@ int main(int argc, char *argv[])
 #endif
 		  if (afd == -1) break;
 		}
-#if MAC_OSX
-	      outbytes = 4096;
-#else
 	      outbytes = curframes * out_chans * sample_size;
-#endif
 	      mus_audio_write(afd, (char *)obuf, outbytes);
 	    }
 	  if (afd != -1) mus_audio_close(afd);
@@ -300,23 +255,23 @@ int main(int argc, char *argv[])
 	   */
 	  buffer_size = 256;   /* 128 probably better */
 	  outbytes = buffer_size * 2 * 2; 
-	  qbufs = (MUS_SAMPLE_TYPE **)CALLOC(chans, sizeof(MUS_SAMPLE_TYPE *));
-	  for (i = 0; i < chans; i++) qbufs[i] = (MUS_SAMPLE_TYPE *)CALLOC(buffer_size, sizeof(MUS_SAMPLE_TYPE));
+	  qbufs = (mus_sample_t **)CALLOC(chans, sizeof(mus_sample_t *));
+	  for (i = 0; i < chans; i++) qbufs[i] = (mus_sample_t *)CALLOC(buffer_size, sizeof(mus_sample_t));
 	  obuf0 = (short *)CALLOC(buffer_size * 2, sizeof(short));
 	  obuf1 = (short *)CALLOC(buffer_size * 2, sizeof(short));
-	  for (i = 0; i < frames; i += buffer_size)
+	  for (m = 0; m < frames; m += buffer_size)
 	    {
-	      if ((i+buffer_size) <= frames)
+	      if ((m + buffer_size) <= frames)
 		curframes = buffer_size;
-	      else curframes = frames - i;
+	      else curframes = frames - m;
 	      mus_sound_read(fd, 0, curframes - 1, chans, qbufs); 
 	      val[0] = 1.0;
 	      for (k = 0, n = 0; k < buffer_size; k++, n += 2) 
 		{
 		  obuf0[n] = MUS_SAMPLE_TO_SHORT(qbufs[0][k]); 
-		  obuf0[n+1] = MUS_SAMPLE_TO_SHORT(qbufs[1][k]);
+		  obuf0[n + 1] = MUS_SAMPLE_TO_SHORT(qbufs[1][k]);
 		  obuf1[n] = MUS_SAMPLE_TO_SHORT(qbufs[2][k]); 
-		  obuf1[n+1] = MUS_SAMPLE_TO_SHORT(qbufs[3][k]);
+		  obuf1[n + 1] = MUS_SAMPLE_TO_SHORT(qbufs[3][k]);
 		}
 	      if (afd0 == -1)
 		{
@@ -339,7 +294,7 @@ int main(int argc, char *argv[])
   return(0);
 }
 
-#else
+#if HAVE_ALSA
 
 /* HAVE_ALSA here
    there should be a command line argument to control this,
@@ -349,10 +304,11 @@ static int use_one_device = 1;
 
 #define MAX_SLOTS 64
 
-int main(int argc, char *argv[])
+static int main_alsa(int argc, char *argv[])
 {
-  int fd, i, chans, srate, frames;
-  MUS_SAMPLE_TYPE **read_bufs;
+  int fd, i, chans, srate;
+  off_t frames, ioff;
+  mus_sample_t **read_bufs;
   int afd[MAX_SLOTS];
   short *out_buf[MAX_SLOTS];
   float val[MAX_SLOTS];
@@ -372,18 +328,14 @@ int main(int argc, char *argv[])
   int min_chans[MAX_SLOTS];
   int max_chans[MAX_SLOTS];
   int alloc_chans;
-  if (argc == 1) 
-    {
-      printf("usage: sndplay file\n"); 
-      exit(0);
-    }
-  mus_sound_initialize();
+  off_t start = 0, end = 0;
+  double begin_time = 0.0, end_time = 0.0;
 
   /* for clisp-based clm use, we need a couple added switches
    * -describe => call mus_audio_describe and exit
    * -buffers axb => set OSS fragment numbers 
    */
-  for (i = 1; i < argc; )
+  for (i = 1; i < argc; i++)
     {
       if (strcmp(argv[i], "-describe") == 0)
 	{
@@ -391,15 +343,28 @@ int main(int argc, char *argv[])
 	  exit(0);
 	}
       else 
-	if (strcmp(argv[i], "-buffers") == 0) 
-	  {
-	    set_buffers(argv[i+1]); 
-	    i++;
-	  }
-      else name = argv[i];
-      i++;
-    }
-
+	{
+	  if (strcmp(argv[i], "-buffers") == 0) 
+	    {
+	      set_buffers(argv[i+1]); 
+	      i++;
+	    }
+	  else
+	    {
+	      if (strcmp(argv[i], "-start") == 0) 
+		{
+		  begin_time = atof(argv[i + 1]);
+		  i++;
+		}
+	      else
+		{
+		  if (strcmp(argv[i], "-end") == 0) 
+		    {
+		      end_time = atof(argv[i + 1]);
+		      i++;
+		    }
+		  else name = argv[i];
+		}}}}
   afd0 = -1;
   afd1 = -1;
   if (!(MUS_HEADER_TYPE_OK(mus_sound_header_type(name))))
@@ -532,43 +497,51 @@ int main(int argc, char *argv[])
 	    }
 	}
       srate = mus_sound_srate(name);
-      frames = mus_sound_samples(name)/chans;
+      frames = mus_sound_frames(name);
       base = 0;
+      start = (off_t)(begin_time * srate);
+      if (start > 0)
+	mus_sound_seek_frame(fd, start);
+      if (end_time > 0.0)
+	end = (off_t)(end_time * srate);
+      else end = frames;
+      if ((end - start) < frames)
+	frames = end - start;
       /* allocate the list of read buffers, each buffer will hold one channel
-	 of the input soundfile, each sample is going to be MUS_SAMPLE_TYPE
+	 of the input soundfile, each sample is going to be mus_sample_t
       */
-      read_bufs = (MUS_SAMPLE_TYPE **)CALLOC(alloc_chans, sizeof(MUS_SAMPLE_TYPE *));
+      read_bufs = (mus_sample_t **)CALLOC(alloc_chans, sizeof(mus_sample_t *));
       for (d = 0; d < allocated; d++)
 	{
 	  int dev = out_devs[d];
 	  for (i = 0; i < out_chans[d]; i++) 
-	    read_bufs[base + i] = (MUS_SAMPLE_TYPE *)CALLOC(samples_per_chan, sizeof(MUS_SAMPLE_TYPE));
+	    read_bufs[base + i] = (mus_sample_t *)CALLOC(samples_per_chan, sizeof(mus_sample_t));
 	  base += out_chans[d];
-	  out_bytes[dev] = samples_per_chan * out_chans[d] * mus_data_format_to_bytes_per_sample(out_format[dev]);
+	  out_bytes[dev] = samples_per_chan * out_chans[d] * mus_bytes_per_sample(out_format[dev]);
 	  out_buf[dev] = (short *)CALLOC(out_bytes[dev], 1);
 	}
-      for (i = 0; i < frames; i += samples_per_chan)
+      for (ioff = 0; ioff < frames; ioff += samples_per_chan)
 	{
-	  MUS_SAMPLE_TYPE **dev_bufs = read_bufs;
-	  if ((i + samples_per_chan) <= frames)
+	  mus_sample_t **dev_bufs = read_bufs;
+	  if ((ioff + samples_per_chan) <= frames)
 	    curframes = samples_per_chan;
 	  else 
 	    {
-	      curframes = frames - i;
+	      curframes = frames - ioff;
 	      for (d = 0; d < allocated; d++)
 		{
 		  int f, dev = out_devs[d];
 #if 1
 		  /* try to kludge around an ALSA bug... */
 		  for (f = 0; f < chans; f++) 
-		    memset(read_bufs[f], 0, samples_per_chan * sizeof(MUS_SAMPLE_TYPE));
+		    memset(read_bufs[f], 0, samples_per_chan * sizeof(mus_sample_t));
 #endif
-		  out_bytes[dev] = curframes * out_chans[d] * mus_data_format_to_bytes_per_sample(out_format[dev]);
+		  out_bytes[dev] = curframes * out_chans[d] * mus_bytes_per_sample(out_format[dev]);
 		}
 	    }
 	  mus_sound_read(fd, 0, curframes - 1, chans, read_bufs); 
 	  /* some systems are happier if we read the file before opening the dac */
-	  /* at this point the data is in separate arrays of MUS_SAMPLE_TYPE */
+	  /* at this point the data is in separate arrays of mus_sample_t */
 	  for (d = 0; d < allocated; d++)
 	    {
 	      int dev = out_devs[d];
@@ -608,5 +581,29 @@ int main(int argc, char *argv[])
   return(0);
 }
 
-
 #endif
+
+int main(int argc, char *argv[])
+{
+#if MACOS
+  argc = ccommand(&argv);
+#endif
+
+  if (argc == 1) 
+    {
+      printf("usage: sndplay file\n"); 
+      exit(0);
+    }
+  mus_sound_initialize();
+ 
+#if HAVE_ALSA
+  if (mus_audio_api() == ALSA_API)
+    {
+      main_alsa(argc, argv);
+      return(0);
+    }
+#endif
+  main_not_alsa(argc, argv);
+  return(0);
+}
+
