@@ -2,14 +2,11 @@
 /* cc -O -DLINUX io.o headers.o audio.o sound.o -o sndsine sndsine.c -lm */
 /* sndsine test.snd */
 
-#if defined(HAVE_CONFIG_H)
-  #include "config.h"
-#endif
-
+#include <config.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#if (defined(NEXT) || (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H))))
+#if (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H)))
   #include <libc.h>
 #else
   #if (!(defined(_MSC_VER))) && (!(defined(MPW_C)))
@@ -27,18 +24,11 @@
 
 #define BUFFER_SIZE 4096
 
-#ifdef DEBUG_MEMORY
-void *mem_calloc(size_t len, size_t size, const char *func, const char *file, int line) {return(calloc(len, size));}
-void *mem_malloc(size_t len, const char *func, const char *file, int line) {return(malloc(len));}
-void mem_free(void *ptr, const char *func, const char *file, int line) {free(ptr);}
-void *mem_realloc(void *ptr, size_t size, const char *func, const char *file, int line) {return(realloc(ptr, size));}
-#endif
-
 int main(int argc, char *argv[])
 {
   int fd, i, k, frames;
   float phase, incr;
-  MUS_SAMPLE_TYPE *obuf[1];
+  mus_sample_t *obuf[1];
 #if MACOS
   argc = ccommand(&argv);
 #endif
@@ -49,8 +39,8 @@ int main(int argc, char *argv[])
     {
       frames = 22050;
       phase = 0.0;
-      incr = TWO_PI*440.0/22050.0;
-      obuf[0] = (MUS_SAMPLE_TYPE *)CALLOC(BUFFER_SIZE, sizeof(MUS_SAMPLE_TYPE));
+      incr = TWO_PI * 440.0 / 22050.0;
+      obuf[0] = (mus_sample_t *)CALLOC(BUFFER_SIZE, sizeof(mus_sample_t));
       k = 0;
       for (i = 0; i < frames; i++)
 	{
@@ -59,12 +49,12 @@ int main(int argc, char *argv[])
 	  k++;
 	  if (k == BUFFER_SIZE)
 	    {
-	      mus_sound_write(fd, 0, BUFFER_SIZE-1, 1, obuf);
+	      mus_sound_write(fd, 0, BUFFER_SIZE - 1, 1, obuf);
 	      k = 0;
 	    }
 	}
-      if (k > 0) mus_sound_write(fd, 0, k-1, 1, obuf);
-      mus_sound_close_output(fd, 22050*mus_data_format_to_bytes_per_sample(MUS_BSHORT));
+      if (k > 0) mus_sound_write(fd, 0, k - 1, 1, obuf);
+      mus_sound_close_output(fd, 22050 * mus_bytes_per_sample(MUS_BSHORT));
       FREE(obuf[0]);
     }
   return(0);
