@@ -1,8 +1,6 @@
 /* sound.c */
 
-#if defined(HAVE_CONFIG_H)
-  #include "config.h"
-#endif
+#include <config.h>
 
 #if USE_SND
   #include "snd.h"
@@ -17,18 +15,19 @@
 #include <stdlib.h>
 #include <stddef.h>
 
-#if (defined(NEXT) || (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H))))
+#if (defined(HAVE_LIBC_H) && (!defined(HAVE_UNISTD_H)))
   #include <libc.h>
 #else
   #if (!(defined(_MSC_VER))) && (!(defined(MPW_C)))
     #include <unistd.h>
   #endif
-  #if (!defined(HAVE_CONFIG_H)) || (defined(HAVE_STRING_H))
+  #if HAVE_STRING_H
     #include <string.h>
   #endif
 #endif
 
 #include "sndlib.h"
+#include "sndlib-strings.h"
 
 #if MACOS
   #ifdef MPW_C
@@ -59,10 +58,13 @@ mus_error_handler_t *mus_error_set_handler(mus_error_handler_t *new_error_handle
 #define MUS_ERROR_BUFFER_SIZE 1024
 static char *mus_error_buffer = NULL;
 
-void mus_error(int error, const char *format, ...)
+int mus_error(int error, const char *format, ...)
 {
 #if HAVE_VPRINTF
   va_list ap;
+#endif
+  if (format == NULL) return(MUS_ERROR); /* else bus error in Mac OSX */
+#if HAVE_VPRINTF
   if (mus_error_buffer == NULL)
     mus_error_buffer = (char *)CALLOC(MUS_ERROR_BUFFER_SIZE, sizeof(char));
   va_start(ap, format);
@@ -84,6 +86,7 @@ void mus_error(int error, const char *format, ...)
     fprintf(stderr, format);
   else fprintf(stderr, "error: %d %s\n", error, format);
 #endif
+  return(MUS_ERROR);
 }
 
 static mus_print_handler_t *mus_print_handler = NULL;
@@ -123,26 +126,28 @@ void mus_print(const char *format, ...)
 }
 
 static const char *mus_initial_error_names[] = {
-  "no error", "no frequency method", "no phase method", "no gen method", "no length method",
+  "no error", "no frequency method", "no phase method", "null gen arg to method", "no length method",
   "no free method", "no describe method", "no data method", "no scaler method",
   "memory allocation failed", "unstable two pole error",
   "can't open file", "no sample input", "no sample output",
   "no such channel", "no file name provided", "no location method", "no channel method",
   "no such fft window", "unsupported data format", "header read failed",
-  "header has too many auxiliary comments", "unsupported header type",
-  "file descriptors not initialized", "not a sound file", "file closed", "write error",
-  "bogus free", "buffer overflow", "buffer underflow", "file overflow", "exponent overflow",
-  "header write failed", "cant open temp file", "interrupted",
+  "unsupported header type", "file descriptors not initialized", "not a sound file", "file closed", "write error",
+  "bogus free", "buffer overflow", "buffer underflow", "file overflow",
+  "header write failed", "can't open temp file", "interrupted", "bad envelope",
 
   "audio channels not available", "audio srate not available", "audio format not available",
-  "no audio input available", "no audio output available", "audio input busy", "audio output busy",
-  "audio configuration not available", "audio input closed", "audio output closed", "audio io interrupted",
+  "no audio input available", "audio configuration not available", 
   "no audio lines available", "audio write error", "audio size not available", "audio device not available",
-  "can't close audio", "can't open audio", "audio read error", "audio amp not available", "audio no op",
+  "can't close audio", "can't open audio", "audio read error", "audio amp not available",
   "can't write audio", "can't read audio", "no audio read permission", 
   "can't close file", "arg out of range",
 
-  "midi open error", "midi read error", "midi write error", "midi close error", "midi init error", "midi misc error"
+  "midi open error", "midi read error", "midi write error", "midi close error", "midi init error", "midi misc error",
+
+  "no channels method", "no hop method", "no width method", "no file-name method", "no ramp method", "no run method",
+  "no increment method", "no inspect method", "no offset method",
+  "no xcoeff method", "no ycoeff method", "no xcoeffs method", "no ycoeffs method", 
 };
 
 static char **mus_error_names = NULL;
@@ -165,12 +170,10 @@ int mus_make_error(char *error_name)
 	    }
 	  else
 	    {
-#ifndef MACOS
 	      len = mus_error_names_size;
 	      mus_error_names_size += 8;
 	      mus_error_names = (char **)REALLOC(mus_error_names, mus_error_names_size * sizeof(char *));
 	      for (i = len; i < mus_error_names_size; i++) mus_error_names[i] = NULL;
-#endif
 	    }
 	}
       len = strlen(error_name);
@@ -182,19 +185,23 @@ int mus_make_error(char *error_name)
 
 const char *mus_error_to_string(int err)
 {
-  if (err < MUS_INITIAL_ERROR_TAG)
-    return(mus_initial_error_names[err]);
-  else
+  if (err >= 0)
     {
-      err -= MUS_INITIAL_ERROR_TAG;
-      if ((mus_error_names) && (err < mus_error_names_size))
-	return(mus_error_names[err]);
-      else return("unknown mus error");
+      if (err < MUS_INITIAL_ERROR_TAG)
+	return(mus_initial_error_names[err]);
+      else
+	{
+	  err -= MUS_INITIAL_ERROR_TAG;
+	  if ((mus_error_names) && (err < mus_error_names_size))
+	    return(mus_error_names[err]);
+	}
     }
+  return("unknown mus error");
 }
 
-static void sound_mus_error(int type, char *msg)
+static void default_mus_error(int type, char *msg)
 {
+  /* default error handler simply prints the error message */
   fprintf(stderr, msg);
 }
 
@@ -206,22 +213,22 @@ static time_t local_file_write_date(const char *filename)
   struct stat statbuf;
   int err;
   err = stat(filename, &statbuf);
-  if (err < 0) return(err);
+  if (err < 0) return((time_t)0);
   return((time_t)(statbuf.st_mtime));
 }
 #else
 static int local_file_write_date(const char *filename) {return(1);}
 #endif
 
-static int sndlib_initialized = 0;
+static bool sndlib_initialized = false;
 
 int mus_sound_initialize(void)
 {
   int err = MUS_NO_ERROR;
   if (!sndlib_initialized)
     {
-      sndlib_initialized = 1;
-      mus_error_handler = sound_mus_error;
+      sndlib_initialized = true;
+      mus_error_handler = default_mus_error;
       err = mus_header_initialize();
       if (err == MUS_NO_ERROR) 
 	err = mus_audio_initialize();
@@ -230,10 +237,6 @@ int mus_sound_initialize(void)
   return(MUS_NO_ERROR);
 }
 
-void mus_sound_finalize(void)
-{
-}
-
 int mus_sample_bits(void)
 {
   /* this to check for inconsistent loads */
@@ -247,20 +250,24 @@ int mus_sample_bits(void)
 typedef struct {
   char *file_name;  /* full path -- everything is keyed to this name */
   int table_pos;
-  int *aux_comment_start, *aux_comment_end;
+  off_t *aux_comment_start, *aux_comment_end;
   int *loop_modes, *loop_starts, *loop_ends;
   int markers, base_detune, base_note;
   int *marker_ids, *marker_positions;
-  unsigned int samples, data_location, true_file_length;
+  off_t samples, true_file_length;
+  off_t data_location;
   int srate, chans, header_type, data_format, original_sound_format, datum_size; 
-  int comment_start, comment_end, header_distributed, type_specifier, bits_per_sample, fact_samples, block_align;
+  off_t comment_start, comment_end;
+  int type_specifier, bits_per_sample, block_align, fact_samples;
   time_t write_date;
-  MUS_SAMPLE_TYPE *maxamps;
+  mus_sample_t *maxamps;
+  off_t *maxtimes;
 } sound_file;
 
 static int sound_table_size = 0;
 static sound_file **sound_table = NULL;
 static sound_file *previous_sf = NULL; /* memoized search */
+static int previous_freed_sf = -1;
 
 static void free_sound_file(sound_file *sf)
 {
@@ -268,6 +275,7 @@ static void free_sound_file(sound_file *sf)
   if (sf)
     {
       sound_table[sf->table_pos] = NULL;
+      previous_freed_sf = sf->table_pos;
       if (sf->aux_comment_start) FREE(sf->aux_comment_start);
       if (sf->aux_comment_end) FREE(sf->aux_comment_end);
       if (sf->file_name) FREE(sf->file_name);
@@ -277,6 +285,7 @@ static void free_sound_file(sound_file *sf)
       if (sf->marker_ids) FREE(sf->marker_ids);
       if (sf->marker_positions) FREE(sf->marker_positions);
       if (sf->maxamps) FREE(sf->maxamps);
+      if (sf->maxtimes) FREE(sf->maxtimes);
       FREE(sf);
     }
 }
@@ -284,35 +293,29 @@ static void free_sound_file(sound_file *sf)
 static sound_file *add_to_sound_table(const char *name)
 {
   int i, pos;
-#ifdef MACOS
-  sound_file **ptr;
-#endif
-  pos = -1;
-  for (i = 0; i < sound_table_size; i++)
-    if (sound_table[i] == NULL) 
-      {
-	pos = i;
-	break;
-      }
+  pos = previous_freed_sf;
   if (pos == -1)
     {
-      pos = sound_table_size;
-      sound_table_size += 16;
-      if (sound_table == NULL)
-	sound_table = (sound_file **)CALLOC(sound_table_size, sizeof(sound_file *));
-      else 
+      for (i = 0; i < sound_table_size; i++)
+	if (sound_table[i] == NULL) 
+	  {
+	    pos = i;
+	    break;
+	  }
+      if (pos == -1)
 	{
-#ifdef MACOS
-	  ptr = (sound_file **)CALLOC(sound_table_size, sizeof(sound_file *));
-	  for (i = 0; i < pos; i++) ptr[i] = sound_table[i];
-	  FREE(sound_table);
-	  sound_table = ptr;
-#else
-	  sound_table = (sound_file **)REALLOC(sound_table, sound_table_size * sizeof(sound_file *));
-#endif
-	  for (i = pos; i < sound_table_size; i++) sound_table[i] = NULL;
+	  pos = sound_table_size;
+	  sound_table_size += 16;
+	  if (sound_table == NULL)
+	    sound_table = (sound_file **)CALLOC(sound_table_size, sizeof(sound_file *));
+	  else 
+	    {
+	      sound_table = (sound_file **)REALLOC(sound_table, sound_table_size * sizeof(sound_file *));
+	      for (i = pos; i < sound_table_size; i++) sound_table[i] = NULL;
+	    }
 	}
     }
+  else previous_freed_sf = -1;
   sound_table[pos] = (sound_file *)CALLOC(1, sizeof(sound_file));
   sound_table[pos]->table_pos = pos;
   sound_table[pos]->file_name = (char *)CALLOC(strlen(name) + 1, sizeof(char));
@@ -320,9 +323,24 @@ static sound_file *add_to_sound_table(const char *name)
   return(sound_table[pos]);
 }
 
+int mus_sound_prune(void)
+{
+  int i, pruned = 0;
+  for (i = 0; i < sound_table_size; i++)
+    if ((sound_table[i]) && 
+	(!(mus_file_probe(sound_table[i]->file_name))))
+      {
+	free_sound_file(sound_table[i]);
+	sound_table[i] = NULL;
+	pruned++;
+      }
+  return(pruned);
+}
+
 int mus_sound_forget(const char *name)
 {
-  int i, len, free_name = 0;
+  int i, len;
+  bool free_name = false;
   char *short_name = NULL;
   if (name == NULL) return(MUS_ERROR);
   if (name[0] == '/')
@@ -335,7 +353,7 @@ int mus_sound_forget(const char *name)
   else
     {
       short_name = mus_expand_filename((char *)name);
-      free_name = 1;
+      free_name = true;
     }
   previous_sf = NULL;
   if (name)
@@ -348,7 +366,11 @@ int mus_sound_forget(const char *name)
 	  free_sound_file(sound_table[i]);
 	  sound_table[i] = NULL;
 	}
-  if (free_name) FREE(short_name);
+#if DEBUG_MEMORY || (defined(MPW_C))
+  if (free_name) FREE(short_name); /* copy_string=MALLOC(snd-utils.c) in io.c mus_expand_filename or strdup=MALLOC in MPW */
+#else
+  if (free_name) free(short_name); /* strdup=malloc, presumably */
+#endif
   return(MUS_NO_ERROR);
 }
 
@@ -375,7 +397,7 @@ static sound_file *check_write_date(const char *name, sound_file *sf)
 	      data_size = lseek(chan, 0L, SEEK_END);
 	      sf->true_file_length = data_size;
 	      sf->samples = mus_bytes_to_samples(sf->data_format, data_size);
-	      close(chan);  
+	      CLOSE(chan);  
 	      return(sf);
 	    }
 	  /* otherwise our data base is out-of-date, so clear it out */
@@ -388,7 +410,7 @@ static sound_file *check_write_date(const char *name, sound_file *sf)
 static sound_file *find_sound_file(const char *name)
 {
   int i;
-  /* perhaps we already have the needed data... */
+  /* perhaps we already have the needed data... (90% hit rate here) */
   if ((previous_sf) &&
       (strcmp(previous_sf->file_name, name) == 0) &&
       (previous_sf->write_date == local_file_write_date(name)))
@@ -409,11 +431,12 @@ static void display_sound_file_entry(FILE *fp, const char *name, sound_file *sf)
   int i, lim;
   time_t date;
   char timestr[64];
+  char *comment;
 #ifndef MPW_C
   date = sf->write_date;
   if (date != 0)
     {
-#if (!defined(HAVE_CONFIG_H)) || defined(HAVE_STRFTIME)
+#if HAVE_STRFTIME
       strftime(timestr, 64, "%a %d-%b-%Y %H:%M:%S", localtime(&date));
 #else
       sprintf(timestr, "%d", date);
@@ -423,7 +446,7 @@ static void display_sound_file_entry(FILE *fp, const char *name, sound_file *sf)
 #else
   sprintf(timestr, "(date unknown)");
 #endif
-  fprintf(fp, "  %s: %s, chans: %d, srate: %d, type: %s, format: %s, samps: %d",
+  fprintf(fp, "  %s: %s, chans: %d, srate: %d, type: %s, format: %s, samps: " OFF_TD,
 	  name,
 	  timestr,
 	  sf->chans,
@@ -450,11 +473,21 @@ static void display_sound_file_entry(FILE *fp, const char *name, sound_file *sf)
 	    {
 	      if (i > 1) fprintf(fp, ", ");
 	      fprintf(fp, " %.3f at %.3f ",
-		      MUS_SAMPLE_TO_FLOAT(sf->maxamps[i + 1]),
-		      (float)(sf->maxamps[i]) / (float)(sf->srate));
+		      MUS_SAMPLE_TO_FLOAT(sf->maxamps[i]),
+		      (sf->srate > 0.0) ? (float)((double)(sf->maxtimes[i]) / (double)(sf->srate)) : (float)(sf->maxtimes[i]));
 	    }
 	}
     }
+  if (mus_file_probe(name))
+    {
+      comment = mus_sound_comment(name);
+      if (comment)
+	{
+	  fprintf(fp, "\n      comment: %s", comment);
+	  FREE(comment);
+	}
+    }
+  else fprintf(fp, " [defunct]");
   fprintf(fp, "\n");
 }
 
@@ -469,7 +502,7 @@ void mus_sound_report_cache(FILE *fp)
       sf = sound_table[i];
       if (sf) 
 	{
-	  display_sound_file_entry(fp, sf->file_name, sound_table[i]);
+	  display_sound_file_entry(fp, sf->file_name, sf);
 	  entries++;
 	}
     }
@@ -477,41 +510,36 @@ void mus_sound_report_cache(FILE *fp)
   fflush(fp);
 }
 
-void mus_sound_print_cache(void)
-{
-  mus_sound_report_cache(stdout);
-}
-
-static void fill_sf_record(const char *name, sound_file *sf)
+static sound_file *fill_sf_record(const char *name, sound_file *sf)
 {
   int i;
   sf->data_location = mus_header_data_location();
   sf->samples = mus_header_samples();
   sf->data_format = mus_header_format();
   sf->srate = mus_header_srate();
-  if (sf->srate < 0) sf->srate = 0;
+  /* if (sf->srate < 0) sf->srate = 0; */
   sf->chans = mus_header_chans();
-  if (sf->chans < 0) sf->chans = 0;
-  sf->datum_size = mus_header_data_format_to_bytes_per_sample();
+  /* if (sf->chans < 0) sf->chans = 0; */
+  sf->datum_size = mus_bytes_per_sample(sf->data_format);
   sf->header_type = mus_header_type();
   sf->original_sound_format = mus_header_original_format();
   sf->true_file_length = mus_header_true_length();
   sf->comment_start = mus_header_comment_start();
   sf->comment_end = mus_header_comment_end();
-  if ((sf->header_type == MUS_AIFC) || 
-      (sf->header_type == MUS_AIFF) || 
-      (sf->header_type == MUS_RIFF))
+  if (((sf->header_type == MUS_AIFC) || 
+       (sf->header_type == MUS_AIFF) || 
+       (sf->header_type == MUS_RIFF)) &&
+      (mus_header_aux_comment_start(0) != 0))
 
     {
-      sf->aux_comment_start = (int *)CALLOC(4, sizeof(int));
-      sf->aux_comment_end = (int *)CALLOC(4, sizeof(int));
+      sf->aux_comment_start = (off_t *)CALLOC(4, sizeof(off_t));
+      sf->aux_comment_end = (off_t *)CALLOC(4, sizeof(off_t));
       for (i = 0; i < 4; i++)
 	{
 	  sf->aux_comment_start[i] = mus_header_aux_comment_start(i);
 	  sf->aux_comment_end[i] = mus_header_aux_comment_end(i);
 	}
     }
-  sf->header_distributed = mus_header_distributed();
   sf->type_specifier = mus_header_type_specifier();
   sf->bits_per_sample = mus_header_bits_per_sample();
   sf->fact_samples = mus_header_fact_samples();
@@ -542,30 +570,15 @@ static void fill_sf_record(const char *name, sound_file *sf)
     }
   /* aux comments */
   previous_sf = sf;
-}
-
-static sound_file *read_sound_file_header_with_fd(int fd, const char *arg)
-{
-  int err = 0;
-  sound_file *sf = NULL;
-  mus_sound_initialize();
-  err = mus_header_read_with_fd(fd);
-  if (err == MUS_ERROR) return(NULL);
-  sf = add_to_sound_table(arg);
-  fill_sf_record(arg, sf);
   return(sf);
 }
 
-static sound_file *read_sound_file_header_with_name(const char *name)
+static sound_file *read_sound_file_header(const char *name)
 {
-  sound_file *sf = NULL;
   mus_sound_initialize();
   if (mus_header_read(name) != MUS_ERROR)
-    {
-      sf = add_to_sound_table(name);
-      fill_sf_record(name, sf);
-    }
-  return(sf);
+    return(fill_sf_record(name, add_to_sound_table(name)));
+  return(NULL);
 }
 
 static sound_file *getsf(const char *arg) 
@@ -573,7 +586,7 @@ static sound_file *getsf(const char *arg)
   sound_file *sf = NULL;
   if (arg == NULL) return(NULL);
   if ((sf = find_sound_file(arg)) == NULL)
-    return(read_sound_file_header_with_name(arg));
+    return(read_sound_file_header(arg));
   return(sf);
 }
 
@@ -583,23 +596,22 @@ static sound_file *getsf(const char *arg)
   if (sf) return(Expression); \
   return(MUS_ERROR)
 
-int mus_sound_samples (const char *arg)         {MUS_SF(arg, sf->samples);}
-int mus_sound_frames (const char *arg)          {MUS_SF(arg, (sf->samples / sf->chans));}
+off_t mus_sound_samples (const char *arg)       {MUS_SF(arg, sf->samples);}
+off_t mus_sound_frames (const char *arg)        {MUS_SF(arg, (sf->chans > 0) ? (sf->samples / sf->chans) : 0);}
 int mus_sound_datum_size (const char *arg)      {MUS_SF(arg, sf->datum_size);}
-int mus_sound_data_location (const char *arg)   {MUS_SF(arg, sf->data_location);}
+off_t mus_sound_data_location (const char *arg) {MUS_SF(arg, sf->data_location);}
 int mus_sound_chans (const char *arg)           {MUS_SF(arg, sf->chans);}
 int mus_sound_srate (const char *arg)           {MUS_SF(arg, sf->srate);}
 int mus_sound_header_type (const char *arg)     {MUS_SF(arg, sf->header_type);}
 int mus_sound_data_format (const char *arg)     {MUS_SF(arg, sf->data_format);}
 int mus_sound_original_format (const char *arg) {MUS_SF(arg, sf->original_sound_format);}
-int mus_sound_comment_start (const char *arg)   {MUS_SF(arg, sf->comment_start);}
-int mus_sound_comment_end (const char *arg)     {MUS_SF(arg, sf->comment_end);}
-int mus_sound_length (const char *arg)          {MUS_SF(arg, sf->true_file_length);}
+off_t mus_sound_comment_start (const char *arg) {MUS_SF(arg, sf->comment_start);}
+off_t mus_sound_comment_end (const char *arg)   {MUS_SF(arg, sf->comment_end);}
+off_t mus_sound_length (const char *arg)        {MUS_SF(arg, sf->true_file_length);}
 int mus_sound_fact_samples (const char *arg)    {MUS_SF(arg, sf->fact_samples);}
-int mus_sound_distributed (const char *arg)     {MUS_SF(arg, sf->header_distributed);}
-int mus_sound_write_date (const char *arg)      {MUS_SF(arg, sf->write_date);}
+int mus_sound_write_date (const char *arg)      {MUS_SF(arg, (int)(sf->write_date));}
 int mus_sound_type_specifier (const char *arg)  {MUS_SF(arg, sf->type_specifier);}
-int mus_sound_align (const char *arg)           {MUS_SF(arg, sf->block_align);}
+int mus_sound_block_align (const char *arg)     {MUS_SF(arg, sf->block_align);}
 int mus_sound_bits_per_sample (const char *arg) {MUS_SF(arg, sf->bits_per_sample);}
 
 float mus_sound_duration(const char *arg) 
@@ -609,8 +621,8 @@ float mus_sound_duration(const char *arg)
   sf = getsf(arg); 
   if (sf) 
     {
-      if ((sf->chans > 0) && (sf->srate > 0.0))
-	val = (float)(sf->samples) / ((float)(sf->chans) * (float)(sf->srate));
+      if ((sf->chans > 0) && (sf->srate > 0))
+	val = (float)((double)(sf->samples) / ((float)(sf->chans) * (float)(sf->srate)));
       else val = 0.0;
     }
   return(val);
@@ -655,45 +667,37 @@ void mus_sound_set_full_loop_info(const char *arg, int *loop)
 	  sf->loop_starts = (int *)CALLOC(2, sizeof(int));
 	  sf->loop_ends = (int *)CALLOC(2, sizeof(int));
 	}
+      sf->loop_modes[0] = loop[6]; 
       if (loop[6] != 0)
 	{
-	  sf->loop_modes[0] = loop[6]; 
 	  sf->loop_starts[0] = loop[0];
 	  sf->loop_ends[0] = loop[1];
 	}
+      else
+	{
+	  sf->loop_starts[0] = 0;
+	  sf->loop_ends[0] = 0;
+	}
+      sf->loop_modes[1] = loop[7];
       if (loop[7] != 0)
 	{
-	  sf->loop_modes[1] = loop[7];
 	  sf->loop_starts[1] = loop[2];
 	  sf->loop_ends[1] = loop[3];
 	}
-      if (loop[4] != 0) sf->base_note = loop[4];
+      else
+	{
+	  sf->loop_starts[1] = 0;
+	  sf->loop_ends[1] = 0;
+	}
+      sf->base_note = loop[4];
       sf->base_detune = loop[5];
     }
 }
 
-void mus_sound_set_loop_info(const char *arg, int *loop)
-{
-  /* backwards compatibility */
-  int bloop[MUS_LOOP_INFO_SIZE];
-  int i;
-  if (loop)
-    {
-      for (i = 0; i < 6; i++) bloop[i] = loop[i];
-      bloop[6] = 1; bloop[7] = 1;
-      mus_sound_set_full_loop_info(arg, bloop);
-    }
-  else mus_sound_set_full_loop_info(arg, NULL);
-}
-
-int mus_sound_aiff_p(const char *arg) 
-{
-  return(mus_sound_header_type(arg) == MUS_AIFF);
-}
-
 char *mus_sound_comment(const char *name)
 {
-  int start, end, fd, len, full_len;
+  off_t start, end, len;
+  int fd, full_len; /* comment string lengths */
   char *sc = NULL, *auxcom;
   sound_file *sf = NULL;
   sf = getsf(name); 
@@ -702,17 +706,21 @@ char *mus_sound_comment(const char *name)
   end = mus_sound_comment_end(name);
   if (end == 0) 
     {
-      if (mus_sound_header_type(name) == MUS_RIFF) 
-	return(mus_header_riff_aux_comment(name, 
-					   sf->aux_comment_start, 
-					   sf->aux_comment_end));
-      if ((mus_sound_header_type(name) == MUS_AIFF) || 
-	  (mus_sound_header_type(name) == MUS_AIFC)) 
-	return(mus_header_aiff_aux_comment(name, 
-					   sf->aux_comment_start, 
-					   sf->aux_comment_end));
+      if (sf->aux_comment_start)
+	{
+	  if (mus_sound_header_type(name) == MUS_RIFF) 
+	    return(mus_header_riff_aux_comment(name, 
+					       sf->aux_comment_start, 
+					       sf->aux_comment_end));
+	  if ((mus_sound_header_type(name) == MUS_AIFF) || 
+	      (mus_sound_header_type(name) == MUS_AIFC)) 
+	    return(mus_header_aiff_aux_comment(name, 
+					       sf->aux_comment_start, 
+					       sf->aux_comment_end));
+	}
       return(NULL);
     }
+  if (end > mus_sound_length(name)) return(NULL);
   len = end - start + 1;
   if (len > 0)
     {
@@ -720,12 +728,12 @@ char *mus_sound_comment(const char *name)
       fd = mus_file_open_read(name);
       if (fd == -1) return(NULL);
       lseek(fd, start, SEEK_SET);
-      sc = (char *)CALLOC(len + 1, sizeof(char)); /* len + 1 calloc'd => we'll always have a trailing null */
+      sc = (char *)CALLOC(len + 1, sizeof(char));
       read(fd, sc, len);
-      close(fd);
-#ifndef MACOS
-      if ((mus_sound_header_type(name) == MUS_AIFF) || 
-	  (mus_sound_header_type(name) == MUS_AIFC)) 
+      CLOSE(fd);
+      if (((mus_sound_header_type(name) == MUS_AIFF) || 
+	   (mus_sound_header_type(name) == MUS_AIFC)) &&
+	  (sf->aux_comment_start))
 	{
 	  auxcom = mus_header_aiff_aux_comment(name, 
 					       sf->aux_comment_start, 
@@ -738,39 +746,38 @@ char *mus_sound_comment(const char *name)
 	      strcat(sc, auxcom);
 	    }
 	}
-#endif
     }
   return(sc);
 }
 
 int mus_sound_open_input (const char *arg) 
 {
-  int fd;
+  int fd = -1;
   sound_file *sf = NULL;
-  mus_sound_initialize();
-  fd = mus_file_open_read(arg);
-  if (fd != MUS_ERROR)
+  if (!(mus_file_probe(arg)))
+    mus_error(MUS_CANT_OPEN_FILE, S_mus_sound_open_input " can't open %s: %s", arg, strerror(errno));
+  else
     {
-      if ((sf = find_sound_file(arg)) == NULL)
-	sf = read_sound_file_header_with_fd(fd, arg);
+      mus_sound_initialize();
+      sf = find_sound_file(arg);
+      if (!sf)
+	sf = read_sound_file_header(arg);
+      if (sf) 
+	{
+	  fd = mus_file_open_read(arg);
+	  mus_file_open_descriptors(fd, arg, sf->data_format, sf->datum_size, sf->data_location, sf->chans, sf->header_type);
+	  lseek(fd, sf->data_location, SEEK_SET);
+	}
     }
-  if (sf) 
-    {
-      mus_file_open_descriptors(fd, arg, sf->data_format, sf->datum_size, sf->data_location, sf->chans, sf->header_type);
-      lseek(fd, sf->data_location, SEEK_SET);
-    }
-  else mus_error(MUS_CANT_OPEN_FILE, "can't open %s: %s\n  [%s[%d] %s]",
-		 arg, strerror(errno),
-		 __FILE__, __LINE__, __FUNCTION__);
   return(fd);
 }
 
 int mus_sound_open_output (const char *arg, int srate, int chans, int data_format, int header_type, const char *comment)
 {
   int fd = MUS_ERROR, err, comlen = 0;
-  mus_sound_forget(arg);
   if (comment) comlen = strlen(comment);
   mus_sound_initialize();
+  mus_sound_forget(arg);
   err = mus_header_write(arg, header_type, srate, chans, 0, 0, data_format, comment, comlen);
   if (err != MUS_ERROR)
     {
@@ -779,7 +786,7 @@ int mus_sound_open_output (const char *arg, int srate, int chans, int data_forma
 	mus_file_open_descriptors(fd,
 				  arg,
 				  data_format,
-				  mus_data_format_to_bytes_per_sample(data_format),
+				  mus_bytes_per_sample(data_format),
 				  mus_header_data_location(),
 				  chans,
 				  header_type);
@@ -787,63 +794,59 @@ int mus_sound_open_output (const char *arg, int srate, int chans, int data_forma
   return(fd);
 }
 
-int mus_sound_reopen_output(const char *arg, int chans, int format, int type, int data_loc)
+int mus_sound_reopen_output(const char *arg, int chans, int format, int type, off_t data_loc)
 {
   int fd;
   mus_sound_initialize();
   fd = mus_file_reopen_write(arg);
   if (fd != -1)
-    mus_file_open_descriptors(fd,
-			      arg,
-			      format,
-			      mus_data_format_to_bytes_per_sample(format),
-			      data_loc,
-			      chans,
-			      type);
+    mus_file_open_descriptors(fd, arg, format, mus_bytes_per_sample(format), data_loc, chans, type);
   return(fd);
 }
 
-int mus_sound_close_input (int fd) 
+int mus_sound_close_input(int fd) 
 {
   return(mus_file_close(fd)); /* this closes the clm file descriptors */
 }
 
-int mus_sound_close_output (int fd, int bytes_of_data) 
+int mus_sound_close_output(int fd, off_t bytes_of_data) 
 {
   char *name;
+  int err = MUS_ERROR, old_type;
   name = mus_file_fd_name(fd);
   if (name)
     {
-      mus_sound_forget(name);
-      mus_header_update_with_fd(fd, mus_file_header_type(fd), bytes_of_data);
-      return(mus_file_close(fd));
+      char *fname;
+      fname = strdup(name); /* strdup defined, if necessary, in io.c */
+      old_type = mus_file_header_type(fd);
+      err = mus_file_close(fd);        /* this frees the original fd->name, so we copied above */
+      /* fd is NULL now */
+      mus_sound_forget(fname);
+      mus_header_change_data_size(fname, old_type, bytes_of_data);
+      free(fname);
+      return(err);
     }
   return(MUS_ERROR);
 }
 
-int mus_sound_read (int fd, int beg, int end, int chans, MUS_SAMPLE_TYPE **bufs) 
+int mus_sound_read(int fd, int beg, int end, int chans, mus_sample_t **bufs) 
 {
   return(mus_file_read(fd, beg, end, chans, bufs));
 }
 
-int mus_sound_write (int tfd, int beg, int end, int chans, MUS_SAMPLE_TYPE **bufs) 
+int mus_sound_write (int tfd, int beg, int end, int chans, mus_sample_t **bufs) 
 {
   return(mus_file_write(tfd, beg, end, chans, bufs));
 }
 
-int mus_sound_seek (int tfd, long offset, int origin) 
-{
-  return(mus_file_seek(tfd, offset, origin));
-}
-
-int mus_sound_seek_frame(int tfd, int frame)
+off_t mus_sound_seek_frame(int tfd, off_t frame)
 {
   return(mus_file_seek_frame(tfd, frame));
 }
 
-enum {SF_CHANS, SF_SRATE, SF_TYPE, SF_FORMAT, SF_LOCATION, SF_SIZE};
+typedef enum {SF_CHANS, SF_SRATE, SF_TYPE, SF_FORMAT, SF_LOCATION, SF_SIZE} sf_field_t;
 
-static int mus_sound_set_field(const char *arg, int field, int val)
+static int mus_sound_set_field(const char *arg, sf_field_t field, int val)
 {
   sound_file *sf; 
   sf = getsf(arg); 
@@ -854,7 +857,22 @@ static int mus_sound_set_field(const char *arg, int field, int val)
 	case SF_CHANS:    sf->chans = val; break;
 	case SF_SRATE:    sf->srate = val; break;
 	case SF_TYPE:     sf->header_type = val; break;
-	case SF_FORMAT:   sf->data_format = val; sf->datum_size = mus_data_format_to_bytes_per_sample(val); break;
+	case SF_FORMAT:   sf->data_format = val; sf->datum_size = mus_bytes_per_sample(val); break;
+	default: return(MUS_ERROR); break;
+	}
+      return(MUS_NO_ERROR);
+    }
+  return(MUS_ERROR);
+}
+
+static int mus_sound_set_off_t_field(const char *arg, sf_field_t field, off_t val)
+{
+  sound_file *sf; 
+  sf = getsf(arg); 
+  if (sf) 
+    {
+      switch (field)
+	{
 	case SF_SIZE:     sf->samples = val; break;
 	case SF_LOCATION: sf->data_location = val; break;
 	default: return(MUS_ERROR); break;
@@ -868,10 +886,10 @@ int mus_sound_set_chans(const char *arg, int val) {return(mus_sound_set_field(ar
 int mus_sound_set_srate(const char *arg, int val) {return(mus_sound_set_field(arg, SF_SRATE, val));}
 int mus_sound_set_header_type(const char *arg, int val) {return(mus_sound_set_field(arg, SF_TYPE, val));}
 int mus_sound_set_data_format(const char *arg, int val) {return(mus_sound_set_field(arg, SF_FORMAT, val));}
-int mus_sound_set_data_location(const char *arg, int val) {return(mus_sound_set_field(arg, SF_LOCATION, val));}
-int mus_sound_set_samples(const char *arg, int val) {return(mus_sound_set_field(arg, SF_SIZE, val));}
+int mus_sound_set_data_location(const char *arg, off_t val) {return(mus_sound_set_off_t_field(arg, SF_LOCATION, val));}
+int mus_sound_set_samples(const char *arg, off_t val) {return(mus_sound_set_off_t_field(arg, SF_SIZE, val));}
 
-int mus_sound_override_header(const char *arg, int srate, int chans, int format, int type, int location, int size)
+int mus_sound_override_header(const char *arg, int srate, int chans, int format, int type, off_t location, off_t size)
 {
   sound_file *sf; 
   /* perhaps once a header has been over-ridden, we should not reset the relevant fields upon re-read? */
@@ -883,7 +901,7 @@ int mus_sound_override_header(const char *arg, int srate, int chans, int format,
       if (format != -1) 
 	{
 	  sf->data_format = format;
-	  sf->datum_size = mus_data_format_to_bytes_per_sample(format);
+	  sf->datum_size = mus_bytes_per_sample(format);
 	}
       if (srate != -1) sf->srate = srate;
       if (chans != -1) sf->chans = chans;
@@ -893,37 +911,61 @@ int mus_sound_override_header(const char *arg, int srate, int chans, int format,
   return(MUS_ERROR);
 }
 
-int mus_sound_maxamp_exists(const char *ifile)
+off_t mus_sound_maxamp(const char *ifile, mus_sample_t *vals)
+{
+  /* for CLM (ffi.lisp) */
+  mus_sample_t *mvals;
+  off_t *mtimes;
+  int chans, i, j;
+  off_t frames;
+  chans = mus_sound_chans(ifile);
+  mvals = (mus_sample_t *)CALLOC(chans, sizeof(mus_sample_t));
+  mtimes = (off_t *)CALLOC(chans, sizeof(off_t));
+  frames = mus_sound_maxamps(ifile, chans, mvals, mtimes);
+  for (i = 0, j = 0; i < chans; i++, j += 2)
+    {
+      vals[j] = (mus_sample_t)(mtimes[i]);
+      vals[j + 1] = mvals[i];
+    }
+  FREE(mvals);
+  FREE(mtimes);
+  return(frames);
+}
+
+bool mus_sound_maxamp_exists(const char *ifile)
 {
   sound_file *sf; 
-  int val = 0;
+  bool val = false;
   sf = getsf(ifile); 
   val = ((sf) && (sf->maxamps));
   return(val);
 }
 
-int mus_sound_maxamp(const char *ifile, MUS_SAMPLE_TYPE *vals)
+off_t mus_sound_maxamps(const char *ifile, int chans, mus_sample_t *vals, off_t *times)
 {
   int ifd, ichans, chn, j;
-  unsigned int n, i, bufnum, frames, curframes;
-  MUS_SAMPLE_TYPE fc;
-  MUS_SAMPLE_TYPE *buffer, *time, *samp;
-  MUS_SAMPLE_TYPE **ibufs;
+  int i, bufnum;
+  off_t n, frames, curframes;
+  mus_sample_t abs_samp;
+  mus_sample_t *buffer, *samp;
+  off_t *time;
+  mus_sample_t **ibufs;
   sound_file *sf; 
   sf = getsf(ifile); 
+  if (sf->chans <= 0) return(MUS_ERROR);
   if ((sf) && (sf->maxamps))
     {
-      for (chn = 0; chn < sf->chans; chn++)
+      if (chans > sf->chans) ichans = sf->chans; else ichans = chans;
+      for (chn = 0; chn < ichans; chn++)
 	{
-	  vals[chn * 2] = sf->maxamps[chn * 2];
-	  vals[chn * 2 + 1] = sf->maxamps[chn * 2 + 1];
+	  times[chn] = sf->maxtimes[chn];
+	  vals[chn] = sf->maxamps[chn];
 	}
       frames = sf->samples / sf->chans;
       return(frames);
     }
   ifd = mus_sound_open_input(ifile);
   if (ifd == MUS_ERROR) return(MUS_ERROR);
-  /* sf = getsf(ifile); */
   ichans = mus_sound_chans(ifile);
   frames = mus_sound_frames(ifile);
   if (frames == 0) 
@@ -932,12 +974,12 @@ int mus_sound_maxamp(const char *ifile, MUS_SAMPLE_TYPE *vals)
       return(0);
     }
   mus_sound_seek_frame(ifd, 0);
-  ibufs = (MUS_SAMPLE_TYPE **)CALLOC(ichans, sizeof(MUS_SAMPLE_TYPE *));
+  ibufs = (mus_sample_t **)CALLOC(ichans, sizeof(mus_sample_t *));
   bufnum = 8192;
   for (j = 0; j < ichans; j++) 
-    ibufs[j] = (MUS_SAMPLE_TYPE *)CALLOC(bufnum, sizeof(MUS_SAMPLE_TYPE));
-  time = (MUS_SAMPLE_TYPE *)CALLOC(ichans, sizeof(MUS_SAMPLE_TYPE));
-  samp = (MUS_SAMPLE_TYPE *)CALLOC(ichans, sizeof(MUS_SAMPLE_TYPE));
+    ibufs[j] = (mus_sample_t *)CALLOC(bufnum, sizeof(mus_sample_t));
+  time = (off_t *)CALLOC(ichans, sizeof(off_t));
+  samp = (mus_sample_t *)CALLOC(ichans, sizeof(mus_sample_t));
   for (n = 0; n < frames; n += bufnum)
     {
       if ((n + bufnum) < frames) 
@@ -946,26 +988,26 @@ int mus_sound_maxamp(const char *ifile, MUS_SAMPLE_TYPE *vals)
       mus_sound_read(ifd, 0, curframes - 1, ichans, ibufs);
       for (chn = 0; chn < ichans; chn++)
 	{
-	  buffer = (MUS_SAMPLE_TYPE *)(ibufs[chn]);
-	  fc = samp[chn];
+	  buffer = (mus_sample_t *)(ibufs[chn]);
 	  for (i = 0; i < curframes; i++) 
-	    if ((buffer[i] > fc) || 
-		(fc < -buffer[i])) 
-	      {
-		time[chn] = i + n; 
-		samp[chn] = buffer[i]; 
-		if (samp[chn] < 0) samp[chn] = -samp[chn];
-		fc = samp[chn];
-	      }
+	    {
+	      abs_samp = mus_sample_abs(buffer[i]);
+	      if (abs_samp > samp[chn])
+		{
+		  time[chn] = i + n; 
+		  samp[chn] = abs_samp;
+		}
+	    }
 	}
     }
   mus_sound_close_input(ifd);
-  for (chn = 0, i = 0; chn < ichans; chn++, i += 2)
+  mus_sound_set_maxamps(ifile, ichans, samp, time); /* save the complete set */
+  if (ichans > chans) ichans = chans;
+  for (chn = 0; chn < ichans; chn++)
     {
-      vals[i] = time[chn];
-      vals[i + 1] = samp[chn];
+      times[chn] = time[chn];
+      vals[chn] = samp[chn];
     }
-  mus_sound_set_maxamp(ifile, vals);
   FREE(time);
   FREE(samp);
   for (j = 0; j < ichans; j++) FREE(ibufs[j]);
@@ -973,70 +1015,80 @@ int mus_sound_maxamp(const char *ifile, MUS_SAMPLE_TYPE *vals)
   return(frames);
 }
 
-int mus_sound_set_maxamp(const char *ifile, MUS_SAMPLE_TYPE *vals)
+int mus_sound_set_maxamps(const char *ifile, int chans, mus_sample_t *vals, off_t *times)
 {
   int i, ichans = 0;
   sound_file *sf; 
   sf = getsf(ifile); 
-  if ((sf) && (sf->maxamps))
+  if (sf)
     {
-      for (i = 0; i < (2 * sf->chans); i++)
-	sf->maxamps[i] = vals[i];
+      if (sf->maxamps)
+	{
+	  if (chans > sf->chans) ichans = sf->chans; else ichans = chans;
+	  for (i = 0; i < ichans; i++)
+	    {
+	      sf->maxtimes[i] = times[i];
+	      sf->maxamps[i] = vals[i];
+	    }
+	}
+      else
+	{
+	  ichans = mus_sound_chans(ifile);
+	  if (sf->maxamps == NULL) 
+	    {
+	      sf->maxamps = (mus_sample_t *)CALLOC(ichans, sizeof(mus_sample_t));
+	      sf->maxtimes = (off_t *)CALLOC(ichans, sizeof(off_t));
+	    }
+	  if (ichans > chans) ichans = chans;
+	  for (i = 0; i < ichans; i++)
+	    {
+	      sf->maxtimes[i] = times[i];
+	      sf->maxamps[i] = vals[i];
+	    }
+	}
+      return(MUS_NO_ERROR);
     }
-  else
-    {
-      ichans = mus_sound_chans(ifile);
-      if (sf->maxamps == NULL) 
-	sf->maxamps = (MUS_SAMPLE_TYPE *)CALLOC(ichans * 2, sizeof(MUS_SAMPLE_TYPE));
-      for (i = 0; i < (2 * ichans); i++)
-	sf->maxamps[i] = vals[i];
-    }
-  return(0);
+  return(MUS_ERROR);
 }
 
-
-int mus_file_to_array(const char *filename, int chan, int start, int samples, MUS_SAMPLE_TYPE *array)
+int mus_file_to_array(const char *filename, int chan, int start, int samples, mus_sample_t *array)
 {
   int ifd, chans, total_read;
-  MUS_SAMPLE_TYPE **bufs;
+  mus_sample_t **bufs;
   ifd = mus_sound_open_input(filename);
   if (ifd == MUS_ERROR) return(MUS_ERROR);
   chans = mus_sound_chans(filename);
   if (chan >= chans) 
     {
       mus_sound_close_input(ifd);      
-      mus_error(MUS_NO_SUCH_CHANNEL, 
-		"can't read %s channel %d to array (file has %d chans)\n [%s[%d] %s]",
-		filename, chan, chans,
-		__FILE__, __LINE__, __FUNCTION__);
-      return(MUS_ERROR);
+      return(mus_error(MUS_NO_SUCH_CHANNEL, "mus_file_to_array can't read %s channel %d (file has %d chans)", filename, chan, chans));
     }
-  bufs = (MUS_SAMPLE_TYPE **)CALLOC(chans, sizeof(MUS_SAMPLE_TYPE *));
+  bufs = (mus_sample_t **)CALLOC(chans, sizeof(mus_sample_t *));
   bufs[chan] = array;
   mus_sound_seek_frame(ifd, start);
-  total_read = mus_file_read_any(ifd, 0, chans, samples, bufs, (MUS_SAMPLE_TYPE *)bufs);
+  total_read = mus_file_read_any(ifd, 0, chans, samples, bufs, (mus_sample_t *)bufs);
   mus_sound_close_input(ifd);
   FREE(bufs);
   return(total_read);
 }
 
-char *mus_array_to_file_with_error(const char *filename, MUS_SAMPLE_TYPE *ddata, int len, int srate, int channels)
+char *mus_array_to_file_with_error(const char *filename, mus_sample_t *ddata, int len, int srate, int channels)
 {
   /* put ddata into a sound file, taking byte order into account */
   /* assume ddata is interleaved already if more than one channel */
   int fd, err = MUS_NO_ERROR;
-  MUS_SAMPLE_TYPE *bufs[1];
+  mus_sample_t *bufs[1];
   mus_sound_forget(filename);
   fd = mus_file_create(filename);
   if (fd == -1) 
     return("mus_array_to_file can't create output file");
   err = mus_file_open_descriptors(fd, filename,
 				  MUS_OUT_FORMAT,
-				  mus_data_format_to_bytes_per_sample(MUS_OUT_FORMAT),
+				  mus_bytes_per_sample(MUS_OUT_FORMAT),
 				  28, channels, MUS_NEXT);
   if (err != MUS_ERROR)
     {
-      err = mus_header_write_next_header(fd, srate, channels, 28, len * sizeof(MUS_SAMPLE_TYPE), MUS_OUT_FORMAT, NULL, 0);
+      err = mus_header_write_next_header(fd, srate, channels, 28, len * sizeof(mus_sample_t), MUS_OUT_FORMAT, NULL, 0);
       if (err != MUS_ERROR)
 	{
 	  bufs[0] = ddata;
@@ -1049,14 +1101,11 @@ char *mus_array_to_file_with_error(const char *filename, MUS_SAMPLE_TYPE *ddata,
   return(NULL);
 }
 
-int mus_array_to_file(const char *filename, MUS_SAMPLE_TYPE *ddata, int len, int srate, int channels)
+int mus_array_to_file(const char *filename, mus_sample_t *ddata, int len, int srate, int channels)
 {
   char *errmsg;
   errmsg = mus_array_to_file_with_error(filename, ddata, len, srate, channels);
   if (errmsg)
-    {
-      mus_error(MUS_CANT_OPEN_FILE, errmsg);
-      return(MUS_ERROR);
-    }
+    return(mus_error(MUS_CANT_OPEN_FILE, errmsg));
   return(MUS_NO_ERROR);
 }
