@@ -13,14 +13,12 @@
 #define S_vct_fillB      "vct-fill!"
 #define S_vct_setB       "vct-set!"
 #define S_vct_mapB       "vct-map!"
-#define S_vct_doB        "vct-do!"
-#define S_vcts_mapB      "vcts-map!"
-#define S_vcts_doB       "vcts-do!"
 #define S_vct_peak       "vct-peak"
 #define S_vct_p          "vct?"
-#define S_list2vct       "list->vct"
-#define S_vct2list       "vct->list"
-#define S_vector2vct     "vector->vct"
+#define S_list_to_vct    "list->vct"
+#define S_vct_to_list    "vct->list"
+#define S_vector_to_vct  "vector->vct"
+#define S_vct_to_vector  "vct->vector"
 #define S_vct_moveB      "vct-move!"
 #define S_vct_subseq     "vct-subseq"
 #define S_vct            "vct"
@@ -32,15 +30,21 @@
 typedef struct {
   int length;
   Float *data;
-  int dont_free;
+  bool dont_free;
 } vct;
 
-void init_vct(void);
-int vct_p(XEN obj);
+void vct_init(void);
+bool vct_p(XEN obj);
 XEN make_vct(int len, Float *data);
 XEN make_vct_wrapper(int len, Float *data);
 vct *get_vct(XEN arg);
 void set_vct_print_length(int len);
+XEN mus_array_to_list(Float *arr, int i, int len);
+char *vct_to_string(vct *v);
+vct *c_make_vct(int len);
+vct *c_free_vct(vct *v);
+vct *c_vct_copy(vct *vc);
+bool vct_equalp(vct *v1, vct *v2);
 
 #define TO_VCT(arg) ((vct *)XEN_OBJECT_REF(arg))
 #define VCT_P(arg) vct_p(arg)
